// ContentsBackupDlg.cpp : implementation file
//

#include "stdafx.h"
#include "khan.h"
#include "ContentsBackupDlg.h"
#include "FileDialog2.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CContentsBackupDlg dialog


CContentsBackupDlg::CContentsBackupDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CContentsBackupDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CContentsBackupDlg)
	m_strFile = _T("");
	m_strBackup = _T("");
	m_strMessage = _T("");
	//}}AFX_DATA_INIT
}


void CContentsBackupDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CContentsBackupDlg)
	DDX_Control(pDX, IDC_BUTTON_BROWSE, m_ctrlBrowseButton);
	DDX_Control(pDX, IDC_BUTTON_BACKUP_CONTENTS, m_ctrlBackupButton);
	DDX_Control(pDX, IDC_EDIT_FILE, m_ctrlFile);
	DDX_Text(pDX, IDC_EDIT_FILE, m_strFile);
	DDX_Text(pDX, IDC_STATIC_BACKUP, m_strBackup);
	DDX_Text(pDX, IDC_STATIC_MSG, m_strMessage);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CContentsBackupDlg, CDialog)
	//{{AFX_MSG_MAP(CContentsBackupDlg)
	ON_BN_CLICKED(IDC_BUTTON_BROWSE, OnButtonBrowse)
	ON_BN_CLICKED(IDC_BUTTON_BACKUP_CONTENTS, OnBackupContents)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CContentsBackupDlg message handlers

BOOL CContentsBackupDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// Set initial header string.
	int nTopics = 0;
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
	{
		m_ctrlBackupButton.EnableWindow(FALSE);
		m_ctrlBrowseButton.EnableWindow(FALSE);
		UpdateData(FALSE);
		return TRUE;
	}

	// Set initial header msg.
	nTopics = pDoc->GetTree().NumTopics();
	m_strBackup.Format(_T("Save the current Contents to a backup file.\n")
					   _T("For example:\n")
					   _T("  C:\\KhanBackups\\KDContents_01.kdc\n")
					   _T("The current Contents contains %d Topics."), nTopics);

	// Lets not backup if other threads running.
	if(pDoc->m_videoPlaying		||
	   pDoc->m_hDownloadThread	||
	   pDoc->m_hAppUpdateThread ||
	   pDoc->m_hUpdateThread	)
	{
		m_strMessage = _T("Please stop any Video playback, Updates, or Downloads first.");
		m_ctrlBackupButton.EnableWindow(FALSE);
		m_ctrlBrowseButton.EnableWindow(FALSE);
		UpdateData(FALSE);
		return TRUE;
	}

	// Set initial message.
	m_strMessage = _T("Press the Browse Button to set a backup file location.");
	UpdateData(FALSE);

	// Disable button till user browses.
	m_ctrlBackupButton.EnableWindow(FALSE);

	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}



// User setting backup path.
void CContentsBackupDlg::OnButtonBrowse() 
{
	// Set dlg.
	CString kdcExtension = _T("kdc");
	CString filters = _T("KD Contents (*.kdc)|*.kdc||");
	CFileDialog2 dlg(FALSE, 
					kdcExtension, 
					_T("KhanContents_01.kdc"),
					OFN_HIDEREADONLY	| 
					OFN_OVERWRITEPROMPT | 
					OFN_NOCHANGEDIR,		// no change cur dir.
					filters,
					NULL );
	dlg.m_ofn.lpstrTitle = _T("Save Contents Backup file");

	// Call.
	if(dlg.DoModal() == IDOK)
	{
		CString path = dlg.GetPathName(); // C:\MyBackups\KhanContents_01.kdc

		// Verify .kdc extension - should be ok as dlg sets.
		CString s = path.Right(4);
		CString s2 = _T(".");
		s2 += kdcExtension;
		if(s != s2)
		{
			// Set message, button, and bail.
			m_strMessage.Format(_T("%s\nhas no .kdc file extension.\n")
								_T("Please enter a new file path."), path);
			m_ctrlBackupButton.EnableWindow(FALSE);
			UpdateData(FALSE);
			return;
		}

		// Update file path text.
		m_strFile = path;

		// Update message.
		m_strMessage = _T("Press the Backup Button to continue.");

		// Enable button.
		m_ctrlBackupButton.EnableWindow(TRUE);

		UpdateData(FALSE);
	}

	// User canceled the file dialog. Set default dlg.
	else
	{
		// Clear any text.
		m_strFile.Empty();

		// Set initial message.
		m_strMessage = _T("Press the Browse Button to set a backup file location.");
		UpdateData(FALSE);

		// Disable button till user re-browses.
		m_ctrlBackupButton.EnableWindow(FALSE);

		UpdateData(FALSE);
	}
}

// Do the backup.
void CContentsBackupDlg::OnBackupContents() 
{
	// Get KhanDB path - same as CTopicTree::Read().
	CString fromPath = ((CApp*)AfxGetApp())->GetAppDataFolder();
	fromPath += _T("\\KhanDB.dat");

	// Backup path should be set.
	CString toPath = m_strFile;

	// Attempt copy.
	if(CopyFile(fromPath, toPath, FALSE))
	{
		// Copied ok - update message.
		m_strMessage = _T("Contents backed up successfully!");

		// Disable backup button.
		m_ctrlBackupButton.EnableWindow(FALSE);

		UpdateData(FALSE);
	}
	// Error
	else
	{
		DWORD err = GetLastError();

		// Format last error.
		LPVOID lpMsgBuf;
		FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
					  FORMAT_MESSAGE_FROM_SYSTEM	 |
					  FORMAT_MESSAGE_IGNORE_INSERTS,
					  NULL,
					  err,
					  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
					  (LPTSTR) &lpMsgBuf,
					  0,
					  NULL);

		// Format err msg.
		m_strMessage.Format(_T("Contents Backup failed.\n%s"), lpMsgBuf);
		m_strMessage.TrimRight();

		// Clean up.
		LocalFree(lpMsgBuf);

		// Disable backup button.
		m_ctrlBackupButton.EnableWindow(FALSE);

		UpdateData(FALSE);
	}	
}
