
/*	Copyright (C) G. Jespersen
	See Licence.txt.*/

// ControlsBar.cpp : implementation file
//

#include "stdafx.h"
#include "Khan.h"
#include "ControlsBar.h"
#include "Doc.h"



#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define HIDE_LEFT _T("Hide Left")
#define SHOW_LEFT _T("Show Left")

/////////////////////////////////////////////////////////////////////////////
// CControlsBar dialog


CControlsBar::CControlsBar()
{
	//{{AFX_DATA_INIT(CControlsBar)
	//}}AFX_DATA_INIT

	// Slider
	m_draggingThumb = 0;
	m_lastVol = 0;
	m_lastTime = 0;
}

CControlsBar::~CControlsBar()
{
	// Set CApp handle
	((CApp*)AfxGetApp())->m_hwndControlBar = NULL;
}

// Override to save Bar Size.
BOOL CControlsBar::Create(CWnd* pParentWnd, UINT nIDTemplate,
						  UINT nStyle, UINT nID)
{
	if(!CDialogBar::Create(pParentWnd,nIDTemplate,nStyle,nID))
		return FALSE;

	m_sizeFloating = m_sizeDocked = m_sizeDefault;

	// Set CApp handle
	((CApp*)AfxGetApp())->m_hwndControlBar = GetSafeHwnd();

	return TRUE;
}

void CControlsBar::DoDataExchange(CDataExchange* pDX)
{
	CDialogBar::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CControlsBar)
	DDX_Control(pDX, IDC_BUTTON_LEFT_PANE, m_ctrlLeftPane);
	DDX_Control(pDX, IDC_STATIC_VOL, m_ctrlStaticVol);
	DDX_Control(pDX, IDC_STATIC_TIME, m_ctrlStaticTime);
	DDX_Control(pDX, IDC_SLIDER_VOL, m_ctrlVol);
	DDX_Control(pDX, IDC_SLIDER_VID, m_ctrlVid);
	DDX_Control(pDX, IDC_BUTTON_STOP, m_ctrlStop);
	DDX_Control(pDX, IDC_BUTTON_PLAY_PAUSE, m_ctrlPause);
	DDX_Control(pDX, IDC_BUTTON_MUTE, m_ctrlMute);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CControlsBar, CDialogBar)
	//{{AFX_MSG_MAP(CControlsBar)
	ON_BN_CLICKED(IDC_BUTTON_MUTE, OnMute)
	ON_BN_CLICKED(IDC_BUTTON_PLAY_PAUSE, OnPlayPause)
	ON_BN_CLICKED(IDC_BUTTON_STOP, OnStop)
	ON_BN_CLICKED(IDC_BUTTON_LEFT_PANE, OnHideLeftPane)
	ON_NOTIFY(NM_RELEASEDCAPTURE, IDC_SLIDER_VID, OnReleasedcaptureSliderVid)
	ON_NOTIFY(NM_RELEASEDCAPTURE, IDC_SLIDER_VOL, OnReleasedcaptureSliderVol)
	ON_WM_HSCROLL()
	ON_WM_MOUSEWHEEL()
	//}}AFX_MSG_MAP

	// Dlg bar special
	ON_MESSAGE(WM_INITDIALOG, OnInitDialog)

	// Video playback messsages from View.
	ON_MESSAGE(WM_USER_BAR_START, OnVidStart)
	ON_MESSAGE(WM_USER_BAR_STOP, OnVidStopped)
	ON_MESSAGE(WM_USER_BAR_PAUSE, OnVidPause)
	ON_MESSAGE(WM_USER_BAR_TIME, OnVidTimeChange)
	ON_MESSAGE(WM_USER_BAR_VOL, OnVidVolChange)
	ON_MESSAGE(WM_USER_BAR_MUTE, OnVidMute)

	// Update handlers for mfc hook up - do nothing.
	ON_UPDATE_COMMAND_UI(IDC_BUTTON_PLAY_PAUSE, OnUpdatePlayPause)
	ON_UPDATE_COMMAND_UI(IDC_BUTTON_STOP, OnUpdateStop)
	ON_UPDATE_COMMAND_UI(IDC_BUTTON_MUTE, OnUpdateMute)
	ON_UPDATE_COMMAND_UI(IDC_SLIDER_VID, OnUpdateProgress)
	ON_UPDATE_COMMAND_UI(IDC_SLIDER_VOL, OnUpdateVolume)

END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CControlsBar message handlers

// Initial display.
LONG CControlsBar::OnInitDialog (UINT wParam, LONG lParam)
{
	TRACE(_T("In CControlsBar::OnInitDialog\n"));

#if VS2005
	// Bug - HandleInitDialog always returns false.
	HandleInitDialog(wParam, lParam);
	UpdateData(FALSE);
#else
	// VC6.0
	// UpdateData is called to subclass or initialize any child controls.
	// See Q185672.
	if ( !HandleInitDialog(wParam, lParam) || !UpdateData(FALSE))
	{
		TRACE0("Warning: UpdateData failed during CControlsBar dialog init.\n");
		return FALSE;
	}
#endif

	// Set texts for no vid playing.
	m_ctrlStaticTime.SetWindowText(_T("Progress"));
	m_ctrlStaticVol.SetWindowText(_T("Volume"));
	m_ctrlPause.SetWindowText(_T("Pause"));
	m_ctrlStop.SetWindowText(_T("Play"));
	m_ctrlMute.SetWindowText(_T("Mute"));

	// Set Left Pane button.
	m_ctrlLeftPane.SetWindowText(HIDE_LEFT);

	// Get ctrl rects.
	m_ctrlVol.GetWindowRect(&m_r1);
	ScreenToClient(&m_r1);
	m_ctrlStaticVol.GetWindowRect(&m_r2);
	ScreenToClient(&m_r2);
	m_ctrlVid.GetWindowRect(&m_r3);
	ScreenToClient(&m_r3);
	m_ctrlStaticTime.GetWindowRect(&m_r4);
	ScreenToClient(&m_r4);
	m_ctrlMute.GetWindowRect(&m_r5);
	ScreenToClient(&m_r5);
	m_ctrlPause.GetWindowRect(&m_r6);
	ScreenToClient(&m_r6);
	m_ctrlStop.GetWindowRect(&m_r7);
	ScreenToClient(&m_r7);
	m_ctrlLeftPane.GetWindowRect(&m_r8);
	ScreenToClient(&m_r8);

	// Disable controls.
	m_ctrlVol.EnableWindow(FALSE);
	m_ctrlVid.EnableWindow(FALSE);
	m_ctrlMute.EnableWindow(FALSE);
	m_ctrlPause.EnableWindow(FALSE);
	m_ctrlStop.EnableWindow(FALSE);
	m_ctrlStaticVol.EnableWindow(FALSE);
	m_ctrlStaticTime.EnableWindow(FALSE);

	m_ctrlLeftPane.EnableWindow(FALSE);// not used
	UpdateData(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


// Called from CPlayerView::ControlsBarEnable()
// when video playback begins.
LRESULT CControlsBar::OnVidStart(WPARAM wParam, LPARAM lParam)	// WM_USER_BAR_START
{
	TRACE(_T("in CControlsBar :OnVidStart\n"));

	// Enable controls.
	m_ctrlVol.EnableWindow(TRUE);
	m_ctrlVid.EnableWindow(TRUE);
	m_ctrlMute.EnableWindow(TRUE);
	m_ctrlPause.EnableWindow(TRUE);
	m_ctrlStop.EnableWindow(TRUE);
	m_ctrlStaticVol.EnableWindow(TRUE);
	m_ctrlStaticTime.EnableWindow(TRUE);

	// Set video slider percent.
	m_ctrlVid.SetRange(0, 1000, TRUE);
	m_ctrlVid.SetPos(0);

	// Set static vid text.
	CString str, strHour = _T("00:"), strMinute = _T("00:"), strSecond = _T("00");
	str.Format(_T("%s%s%s"), strHour, strMinute, strSecond);
	m_ctrlStaticTime.SetWindowText(str);

	// Set Volume slider percent.
	m_lastVol = 50;
	m_ctrlVol.SetRange(0, 100, TRUE);
	m_ctrlVol.SetPos(m_lastVol);

	// Set static vol text.
	CString s;
	s.Format(_T("%d %%"), m_lastVol);
	m_ctrlStaticVol.SetWindowText(s);

	// Set pause button text.
	m_ctrlPause.SetWindowText(_T("Pause"));

	// Set muste button text
	m_ctrlMute.SetWindowText(_T("Mute"));

	// Set stop button text.
	m_ctrlStop.SetWindowText(_T("Stop"));

	UpdateData(FALSE);

	return 0;
}

// Called from CPlayerView::ControlsBarDisable() when playback stops.
LRESULT CControlsBar::OnVidStopped(WPARAM wParam, LPARAM lParam)	// WM_USER_BAR_STOP
{
	TRACE(_T("in CControlsBar OnVidStopped\n"));

	// Zero video ctrl.
	m_ctrlVid.SetPos(0);
	m_ctrlStaticTime.SetWindowText(_T("Progress"));
	m_lastTime = 0;

	// Zero volume ctrl.
	m_ctrlVol.SetPos(0);
	m_ctrlStaticVol.SetWindowText(_T("Volume"));
	m_lastVol = 0;

	// Disable controls.
	m_ctrlVol.EnableWindow(FALSE);
	m_ctrlVid.EnableWindow(FALSE);
	m_ctrlMute.EnableWindow(FALSE);
	m_ctrlPause.EnableWindow(FALSE);
	m_ctrlStop.EnableWindow(FALSE);
	m_ctrlStaticVol.EnableWindow(FALSE);
	m_ctrlStaticTime.EnableWindow(FALSE);

	return 0;
}

// Set Pause button text.
// Called from CPlayerView::ControlsBarDisable() when player is paused / depaused.
LRESULT CControlsBar::OnVidPause(WPARAM wParam, LPARAM lParam)	// WM_USER_BAR_PAUSE
{
	// Vid now Paused.
	if(lParam)
	{
		m_ctrlPause.SetWindowText(_T("Play"));
	}
	// Vid now Playing.
	else
	{
		m_ctrlPause.SetWindowText(_T("Pause"));
	}
	return 0;
}

// Update the slider position and the static text box.
// Called from CPlayerView::OnTimer every .1 seconds or so
// and from CPlayerView::OnPlaybackTime().
LRESULT CControlsBar::OnVidTimeChange(WPARAM wParam, LPARAM lParam)	// WM_USER_BAR_TIME
{
	//TRACE("In CControlsBar OnVidTimeChange\n");

	// Set slider pos - 1000 divs.
	// Note there are not 1000 pix in ctrl width.
	// Steps are one or two pix per rounding.
	// 
	// Pass if user dragging the slider ctrl - occures when 
	// called from CPlayerView::OnPlaybackTime().
	if(!m_draggingThumb)
		m_ctrlVid.SetPos((int)lParam);

	// Save last pos - see OnHScroll().
	m_lastTime = (int)lParam;

	/*dbg test
	CString str;
	str.Format(_T("%d"), (int)lParam);
	m_ctrlStaticTime.SetWindowText(str); //ok - 1000
	return 0;*/

	// Set strings to nearest second;
	SetVideoTimeString((int)wParam);
	return 0;
}

// Set m_ctrlStaticTime str per seconds param.
// Called from OnVidTimeChange() and OnHScroll().
void CControlsBar::SetVideoTimeString(int sec)
{
	// Set static text.
	CString strHour = _T("00:"), strMinute = _T("00:"), strSecond = _T("00");
	int hours = 0, minutes = 0, seconds = 0;

	// Hours.
	if(sec >= (60 * 60) )
	{
		hours = sec / (60 * 60);
		sec -= hours * 60 * 60;
		if(hours < 10)
			strHour.Format(_T("0%d:"), hours);
		else
			strHour.Format(_T("%d:"), hours);
	}
	// Minutes
	if(sec >= 60)
	{
		minutes = sec / 60;
		sec -= minutes * 60;
		if(minutes < 10)
			strMinute.Format(_T("0%d:"), minutes);
		else
			strMinute.Format(_T("%d:"), minutes);
	}
	// Seconds
	if(sec > 0)
	{
		seconds = sec;
		if(seconds < 10)
			strSecond.Format(_T("0%d"), seconds);
		else
			strSecond.Format(_T("%d"), seconds);
	}
	CString str;
	str.Format(_T("%s%s%s"), strHour, strMinute, strSecond);
	m_ctrlStaticTime.SetWindowText(str);
}

// Called from CPlayerView::StepVolume().
LRESULT CControlsBar::OnVidVolChange(WPARAM wParam, LPARAM lParam)	// WM_USER_BAR_VOL
{
	// Bail if dragging the thumb.
	if(m_draggingThumb)
		return 0;

	m_lastVol = (int)lParam;

	// Set slider.
	m_ctrlVol.SetPos(m_lastVol);

	// Set text
	CString str;
	str.Format(_T("%d %%"), m_lastVol);
	m_ctrlStaticVol.SetWindowText(str);

	return 0;
}

// Set Mute button text.
// Called from CPlayerView::ToggleVolumeMute().
LRESULT CControlsBar::OnVidMute(WPARAM wParam, LPARAM lParam)	// WM_USER_BAR_MUTE
{
	// Now muted - set text to "Unmute"
	if(lParam)
		m_ctrlMute.SetWindowText(_T("Unmute"));
	else
		m_ctrlMute.SetWindowText(_T("Mute"));
	return 0;
}

// User is manipulating a slider.
void CControlsBar::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	if(pScrollBar->IsKindOf(RUNTIME_CLASS(CSliderCtrl)) )
    {
		// Video Slider.
		if(pScrollBar->GetSafeHwnd() == m_ctrlVid.GetSafeHwnd() )
		{
			// If paused by user, no time change allowed.
			CString str;
			m_ctrlPause.GetWindowText(str);
			if(str == _T("Play"))
			{
				m_ctrlVid.SetPos(m_lastTime);
				return;
			}

			switch(nSBCode)
			{
				// User clicking on slider.
				case TB_BOTTOM:
				case TB_ENDTRACK:
				case TB_LINEDOWN:
				case TB_LINEUP:
				case TB_PAGEDOWN:
				case TB_PAGEUP:
				case TB_TOP:
				case TB_THUMBTRACK:
				case TB_THUMBPOSITION:
				{
					// Set flag. OnReleasedcaptureSliderVid() resets
					// when mouse is released.
					m_draggingThumb = TRUE;

					// Get slider pos - range is 0 - 1000.
					int pos = m_ctrlVid.GetPos();

					// Call CPlayerView::OnPlaybackTime().
					// This will pause the graph and adjust the pos.
					//
					// -wParam indicates we are moving the slider
					// -lParam is the slider pos
					HWND hwndView = ((CApp*)AfxGetApp())->m_hwndView;
					if(hwndView)
						::PostMessage(hwndView, WM_USER_VID_TIME, (WPARAM)1, (LPARAM)pos);
				}
				default:
					break;
			}
		}
		// Volume Slider.
		else if(pScrollBar->GetSafeHwnd() == m_ctrlVol.GetSafeHwnd() )
		{
			// If muted, no vol change allowed.
			CString str;
			m_ctrlMute.GetWindowText(str);
			if(str == _T("Unmute"))
			{
				m_ctrlVol.SetPos(m_lastVol);
				return;
			}

			// Adjust vol.
			switch(nSBCode)
			{
				case TB_TOP:		// keyboard
				case TB_BOTTOM:
				case TB_LINEDOWN:
				case TB_LINEUP:
				case TB_ENDTRACK:	// mouse or keyboard
				case TB_PAGEDOWN:
				case TB_PAGEUP:
				case TB_THUMBTRACK:	// mouse
				case TB_THUMBPOSITION:
				{
					// Get slider pos. Note range is 0 - 100.
					m_lastVol = m_ctrlVol.GetPos();

					// Set text
					CString str;
					str.Format(_T("%d %%"), m_lastVol);
					m_ctrlStaticVol.SetWindowText(str);

					// Message View.
					HWND hwndView = ((CApp*)AfxGetApp())->m_hwndView;
					if(hwndView)
						::PostMessage(hwndView, WM_USER_VID_VOL, 0, m_lastVol);
					break;
				}
				default:
					break;
			}
		}
    }
	CDialogBar::OnHScroll(nSBCode, nPos, pScrollBar);
}

// User has dragged and released the time slider - see OnHScroll().
// Update the Video position.
void CControlsBar::OnReleasedcaptureSliderVid(NMHDR* pNMHDR, LRESULT* pResult) 
{
	*pResult = 0;

	// If paused by user, no time change allowed.
	CString str;
	m_ctrlPause.GetWindowText(str);
	if(str == _T("Play"))
	{
		m_ctrlVid.SetPos(m_lastTime);
		return;
	}

	m_draggingThumb = FALSE;
	int pos = m_ctrlVid.GetPos();

	// Call CPlayerView::OnPlaybackTime().
	//
	// This call will de-pause the graph, and 
	// set the final position.
	//	wParam indicates we are moving the slider, = 0.
	//	lParam is the slider pos
	HWND hwndView = ((CApp*)AfxGetApp())->m_hwndView;
	if(hwndView)
		::PostMessage(hwndView, WM_USER_VID_TIME, 0, (LPARAM)pos);
}

// Volume Slider - not used - see OnHScroll().
void CControlsBar::OnReleasedcaptureSliderVol(NMHDR* pNMHDR, LRESULT* pResult) 
{
	*pResult = 0;
}

// Mute Button handler.
// Mute unmute video sound.
void CControlsBar::OnMute() 
{
	// Message view. Note View calls back to update controls.
	// See OnVidMute().
	HWND hwndView = ((CApp*)AfxGetApp())->m_hwndView;
	if(hwndView)
		::PostMessage(hwndView, WM_USER_VID_MUTE, 0, 0);
}

// Pause Button handler.
// Play Pause a video.
void CControlsBar::OnPlayPause() 
{
	// Doc ptr.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return;

	// Message view. Note View calls back to update controls.
	// See OnVidXXXX().
	//
	// If no video is playing.
	if(!pDoc->m_videoPlaying)
	{
		// If a video is currently selected, play it.
		if(!pDoc->m_curVideo.mp4.IsEmpty())
		{
			HWND hwndView = ((CApp*)AfxGetApp())->m_hwndView;
			if(hwndView)
				::PostMessage(hwndView, WM_USER_VID_START, 0, 0);
		}
	}
	// Video is playing - toggle play / pause.
	else
	{
		HWND hwndView = ((CApp*)AfxGetApp())->m_hwndView;
		if(hwndView)
			::PostMessage(hwndView, WM_USER_VID_PAUSE, 0, 0);
	}
}

// Stop Button handler (used only for stopping).
// Stop a playing video.
void CControlsBar::OnStop() 
{
	// Message view. Note View calls back to disable controls.
	// See OnVidStopped().
	HWND hwndView = ((CApp*)AfxGetApp())->m_hwndView;
	if(hwndView)
		::PostMessage(hwndView, WM_USER_VID_STOP, 0, 0);
}

// Update handlers needed for mfc hook up.
void CControlsBar::OnUpdatePlayPause(CCmdUI* pCmdUI)
{
	// DocData.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return;

	// If ListFiles showing.
	if(pDoc->m_listFiles.ListShowing())
		pCmdUI->Enable(FALSE);

	// If no video is playing.
	else if(!pDoc->m_videoPlaying)
	{
		// Set to Play.
		m_ctrlPause.SetWindowText(_T("Play"));

		// If a video is currently selected, show the play.
		if(!pDoc->m_curVideo.mp4.IsEmpty())
			pCmdUI->Enable(TRUE);

		// Else disable.
		else
			pCmdUI->Enable(FALSE);
		UpdateData(FALSE);
	}
	// Video is playing.
	else
	{
	//	if(pDoc->m_vlcPlayer.m_paused)
	//		m_ctrlPlayPause.SetWindowText(_T("Play"));
	//	else
	//		m_ctrlPlayPause.SetWindowText(_T("Pause"));
	//	pCmdUI->Enable(TRUE);
	}
}

// Update Stop Button.
void CControlsBar::OnUpdateStop(CCmdUI* pCmdUI)
{
	// DocData.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return;

	// If no video is playing.
	if(!pDoc->m_videoPlaying)
	{
		pCmdUI->Enable(FALSE);
	}
	// Video is playing.
	else
	{
		pCmdUI->Enable(TRUE);
	}
}
void CControlsBar::OnUpdateMute(CCmdUI* pCmdUI)
{}
void CControlsBar::OnUpdateVolume(CCmdUI* pCmdUI)
{}
void CControlsBar::OnUpdateProgress(CCmdUI* pCmdUI)
{}

// Hide Show the left Pane tabs.
void CControlsBar::OnHideLeftPane() 
{
	CString s;
	m_ctrlLeftPane.GetWindowText(s);

	HWND hWnd = ((CApp*)AfxGetApp())->m_hwndSplitter;
	if(hWnd)
	{
		if(s == SHOW_LEFT)
		{
			// Show.
			m_ctrlLeftPane.SetWindowText(HIDE_LEFT);
			::PostMessage(hWnd, WM_USER_HIDE_LEFT_PANE, 0, 0);	
		}
		else if(s == HIDE_LEFT)
		{
			// Hide.
			m_ctrlLeftPane.SetWindowText(SHOW_LEFT);
			::PostMessage(hWnd, WM_USER_HIDE_LEFT_PANE, 0, 1);	
		}
		// Note: CSplitterView::OnHideLeftPane sets foocus to view.
	}
}


///////////////////////////////////////////////////////////////////
// Bar sizing
 
CSize CControlsBar::CalcDynamicLayout(int nLength, DWORD dwMode)
{
	// Note: the HideLeft button is from another app
	// and can be removed.

	CSize newSz = m_sizeDefault;

	// Get the bars docking rect:
	//	- client area if docked.
	//	- bar's bounding rect if floating.
	CRect frmRect;
	CFrameWnd * pFrame = GetDockingFrame();
	pFrame->GetClientRect(&frmRect);

	// Bail if empty
	if(frmRect.IsRectEmpty())
		return newSz;

	// Not used
	if(dwMode & LM_COMMIT)
		int g = 0;	

	// If being dragged to a new location.
	if(dwMode & LM_MRUWIDTH)
	{
		// Dragging a floated bar - stay with existing.
		if(IsFloating())
			return m_sizeFloating;

		// Dragging from a docked pos to a floating or
		// another docked pos - set to default bar.
		else
		{
			CRect r(0, 0, m_sizeDefault.cx, m_sizeDefault.cy);
			CalcLayout(r);
			m_sizeFloating = m_sizeDefault; 
			return m_sizeDefault;
		}
	}

	// If docked.
	else if((dwMode & LM_VERTDOCK) || (dwMode & LM_HORZDOCK))
	{
		// If the bar's left is somewhere right of the client's 
		// left side, offset bar rightward that distance. This
		// makes the bar stick in its dropped position, clipping
		// any controls which are right of the clients right 
		// side.
		WINDOWPLACEMENT wp;
		GetWindowPlacement(&wp);
		if(wp.rcNormalPosition.left > 0 &&
		   frmRect.right > wp.rcNormalPosition.left)
		{
	//		frmRect.right -= wp.rcNormalPosition.left;
		}
		// Recalc.
		if(CalcLayout(frmRect))
			newSz = frmRect.Size();

		m_sizeDocked = newSz;
		return newSz;
	}

	// Else bar has been floated and is being resized.
	else if(nLength)
	{
		// User resizing bar's vert size.
		if(dwMode & LM_LENGTHY)
		{
			// Get calculated rect.
			CRect calcRect = frmRect;
			if(CalcLayout(calcRect))
			{
				// All we do here is ensure the rect is not
				// dragged smaller than the calculated
				// vertical size. Bigger is ok.
				if(nLength > calcRect.bottom)
					m_sizeFloating.cy = nLength;
			}
		}

		// User resizing bar's horz size.
		else
		{
			// Set copy of frame rect to the drag width.
			CRect calcRect = frmRect;
			calcRect.right = nLength;

			// Get calculated rect.
			if(CalcLayout(calcRect))
			{
				// Get newly calculated size.
				newSz = calcRect.Size();

				// User can drag wider than calculated size.
				if(nLength > newSz.cx)
					m_sizeFloating.cx = nLength;
				// But not smaller.
				else
					m_sizeFloating.cx = newSz.cx;

				// If user dragged rect taller than calculated size
				// (in LM_LENGTHY above), retain that height.
				// But if too small, adjust.
				if(newSz.cy > m_sizeFloating.cy)
					m_sizeFloating.cy = newSz.cy;
			}
		}
		// Return revised or not.
		return m_sizeFloating;
	}
	// No nLength.
	else
		return m_sizeFloating;

	// Should not get here.
	return m_sizeDefault;
}

// For now, justify controls right if posssible.
BOOL CControlsBar::CalcLayout(CRect & frmRect)
{
	// Get original ctrl rects - set in OnIntitialDlg().
	CRect r1 = m_r1;
	CRect r2 = m_r2;
	CRect r3 = m_r3;
	CRect r4 = m_r4;
	CRect r5 = m_r5;
	CRect r6 = m_r6;
	CRect r7 = m_r7;
	CRect r8 = m_r8;

	int space = 6;

	// Get width of sliders and texts.
	int widthSliders = r1.Width() + r2.Width()  + r3.Width() + r4.Width() + (space * 4);

	// Get width of buttons.
	int widthButtons = r5.Width() + r6.Width() + r7.Width() + r8.Width() + (space * 5);

	// Get total.
	int widthTotal = widthSliders + widthButtons;

	// Frame width.
	int widthFrame = frmRect.Width();

	// Wrap controls on two rows. Three cases:
	if(widthTotal >= widthFrame)
	{
		int shiftY = m_r1.Height();

		// Case 1:
		// If smaller than the buttons, justify both rows left.
		// Sliders and buttons will be clipped on right.
		if(widthButtons >= widthFrame)
		{
			// Sliders left justified.
			r4.left  = frmRect.left + space;
			r4.right = r4.left  + m_r4.Width();

			r3.left  = r4.right + space;
			r3.right = r3.left  + m_r3.Width();

			r2.left  = r3.right + space;
			r2.right = r2.left  + m_r2.Width();

			r1.left  = r2.right + space;
			r1.right = r1.left  + m_r1.Width();

			// Buttons left justified and down.
			r8.left  = frmRect.left + space;
			r8.right = r8.left  + m_r8.Width();
			r8.OffsetRect(0, shiftY);

			r7.left  = r8.right + space;
			r7.right = r7.left  + m_r7.Width();
			r7.OffsetRect(0, shiftY);

			r6.left  = r7.right + space;
			r6.right = r6.left  + m_r6.Width();
			r6.OffsetRect(0, shiftY);

			r5.left  = r6.right + space;
			r5.right = r5.left  + m_r5.Width();
			r5.OffsetRect(0, shiftY);

			// Set frame.
			frmRect.bottom = r8.bottom + space;
		}

		// Case 2:
		// If smaller than sliders, justify sliders left, buttons right.
		// Sliders will be clipped on right.
		else if(widthSliders >= widthFrame)
		{
			// Sliders left justified.
			r4.left  = frmRect.left + space;
			r4.right = r4.left  + m_r4.Width();

			r3.left  = r4.right + space;
			r3.right = r3.left  + m_r3.Width();

			r2.left  = r3.right + space;
			r2.right = r2.left  + m_r2.Width();

			r1.left  = r2.right + space;
			r1.right = r1.left  + m_r1.Width();

			// Buttons right justified and down.
			r5.right = frmRect.right - space;
			r5.left  = r5.right - m_r5.Width();
			r5.OffsetRect(0, shiftY);

			r6.right = r5.left  - space;
			r6.left  = r6.right - m_r6.Width();
			r6.OffsetRect(0, shiftY);

			r7.right = r6.left  - space;
			r7.left  = r7.right - m_r7.Width();
			r7.OffsetRect(0, shiftY);

			r8.right = r7.left  - space;
			r8.left  = r8.right - m_r8.Width();
			r8.OffsetRect(0, shiftY);

			// Set frame.
			frmRect.bottom = r8.bottom + space;
		}

		// Case 3:
		// Else set sliders and buttons fit - justified right.
		else
		{
			// Sliders right justified.
			r1.right = frmRect.right - space;
			r1.left	 = r1.right - m_r1.Width();

			r2.right = r1.left  - space;
			r2.left  = r2.right - m_r2.Width();

			r3.right = r2.left  - space;
			r3.left  = r3.right - m_r3.Width();

			r4.right = r3.left  - space;
			r4.left  = r4.right - m_r4.Width();

			// Buttons right justified and down.
			r5.right = frmRect.right - space;
			r5.left  = r5.right - m_r5.Width();
			r5.OffsetRect(0, shiftY);

			r6.right = r5.left  - space;
			r6.left  = r6.right - m_r6.Width();
			r6.OffsetRect(0, shiftY);

			r7.right = r6.left  - space;
			r7.left  = r7.right - m_r7.Width();
			r7.OffsetRect(0, shiftY);

			r8.right = r7.left  - space;
			r8.left  = r8.right - m_r8.Width();
			r8.OffsetRect(0, shiftY);

			// Set frame.
			frmRect.bottom = r8.bottom + space;
		}
	}

	// No wrap - justify right - single row.
	else
	{
		r1.right = frmRect.right - space;
		r1.left	 = r1.right - m_r1.Width();

		r2.right = r1.left  - space;
		r2.left  = r2.right - m_r2.Width();

		r3.right = r2.left  - space;
		r3.left  = r3.right - m_r3.Width();

		r4.right = r3.left  - space;
		r4.left  = r4.right - m_r4.Width();

		r5.right = r4.left  - space;
		r5.left  = r5.right - m_r5.Width();

		r6.right = r5.left  - space;
		r6.left  = r6.right - m_r6.Width();

		r7.right = r6.left  - space;
		r7.left  = r7.right - m_r7.Width();

		r8.right = r7.left  - space;
		r8.left  = r8.right - m_r8.Width();

		// Set frame.
		frmRect.bottom = r8.bottom + space;
	}

	// Position rects.
	m_ctrlVol.MoveWindow(r1, TRUE);
	m_ctrlStaticVol.MoveWindow(r2, TRUE);
	m_ctrlVid.MoveWindow(r3, TRUE);
	m_ctrlStaticTime.MoveWindow(r4, TRUE);
	m_ctrlMute.MoveWindow(r5, TRUE);
	m_ctrlPause.MoveWindow(r6, TRUE);
	m_ctrlStop.MoveWindow(r7, TRUE);
	m_ctrlLeftPane.MoveWindow(r8, TRUE);

	return TRUE;
}







	






