
/*	Copyright (C) G. Jespersen
	See Licence.txt.*/

// 
// File not compatable with VLC version
//

// DataView.cpp : implementation of the CDataView class
// DirectShow version
//

#include "stdafx.h"
#include "Khan.h"

#include "Doc.h"
#include "DataView.h"
#include "JsonParse.h"
#include "math.h"
#include "MainFrm.h"
#include "VideoStoreDlg.h"
#include "FontSetup.h"
#include "KhanInet.h"

// test dump - see CApp::Run() - v0.9.2.0
#include "Dump.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Vid pos timer.
#define TIMER_VID_ID	1

// Response timer
//#define RESPONSE_TIMER	2 // 12/13/14



/////////////////////////////////////////////////////////////////////////////
// CDataView

IMPLEMENT_DYNCREATE(CDataView, CScrollView)

BEGIN_MESSAGE_MAP(CDataView, CScrollView)
	//{{AFX_MSG_MAP(CDataView)
	ON_WM_SIZE()
	ON_WM_MOUSEWHEEL()
	ON_COMMAND(ID_SETUP_FONT_RIGHT, OnSetupFont)
	ON_WM_KEYDOWN()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_RBUTTONDOWN()
	ON_WM_TIMER()
	ON_WM_ERASEBKGND()
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_UPDATE_COMMAND_UI(ID_FILE_OPEN, OnUpdateFileOpen)
	//}}AFX_MSG_MAP

	// Video callback
	ON_MESSAGE(WM_USER_VIDEOEVENT, OnVideoEvent)

	// Activation
	ON_MESSAGE(WM_USER_SET_ACTIVATE, OnSetActivate)

	// From ControlsBar
	ON_MESSAGE(WM_USER_VID_START, OnPlaybackStart)
	ON_MESSAGE(WM_USER_VID_STOP,  OnPlaybackStop)
	ON_MESSAGE(WM_USER_VID_PAUSE, OnPlaybackPause)
	ON_MESSAGE(WM_USER_VID_MUTE,  OnPlaybackMute)
	ON_MESSAGE(WM_USER_VID_TIME,  OnPlaybackTime)
	ON_MESSAGE(WM_USER_VID_VOL,	  OnPlaybackVol)

	// Testing - save
	ON_COMMAND(ID_TEST, OnTest)

END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDataView construction/destruction

CDataView::CDataView()
{
	// DirectShow
	m_redrawVid = FALSE;
	m_isKhanVideo = FALSE;

	// Draw text.
	m_formatText = TRUE;
	m_textOffset.cx = 10;
	m_textOffset.cy = 0;

	// Scrolling
	m_scrollResizing = FALSE;

	// View
	m_vidBkgColor = RGB(255, 255, 255);//255, 251, 240 win cream, 214, 214, 214 light grey
}

CDataView::~CDataView()
{
	TRACE(_T("Destructing CDataView\n"));

	// Set CApp handle.
	CApp * pApp = (CApp *)AfxGetApp();
	if(pApp)
	{
		pApp->m_hwndView = NULL;
	}
}

void CDataView::OnInitialUpdate() 
{
	CScrollView::OnInitialUpdate();

	// Set bars.
	SetScrollViewSize();	
	
	// Set app handle.
	CApp * pApp = (CApp *)AfxGetApp();
	if(pApp)
		pApp->m_hwndView = m_hWnd;	
}

// Set the height for the vert scroll bar.
void CDataView::SetScrollViewSize()
{
	CDoc* pDoc = GetDocument();
	if(!pDoc)
		return;

	CSize sz(0, 0);

	// If a video is playing, hide the bars.
	if(m_player.IsRunningOrPaused())
	{
		SetScrollSizes(MM_TEXT, sz); // zero sz ok
		return;
	}

	// Set up font.
	if(!(HFONT)m_font)
	{
		LOGFONT lf = pDoc->m_viewLf;
		m_font.CreateFontIndirect(&lf);
	}

	// Calc minimum view height.
	CClientDC dc(this);
	CFont * old = dc.SelectObject(&m_font);
	CSize strSz = dc.GetTextExtent(_T("X")); 
	dc.SelectObject(old);
	sz.cy = strSz.cy * m_strAr.GetSize();			// v0.9.4 - blank line added to top of array

	// Set CScrollView.
	SetScrollSizes(MM_TEXT, sz);

	// Scroll to the top if not resizing.			// v0.9.4 see OnSize()
	if(!m_scrollResizing)
	{
		POINT p = {0, 0};
		ScrollToPosition(p);
	}
	// Else update exposed view area.
	// Scroll pos remains the same value.
	else
	{
		// Reset flag.
		m_scrollResizing = FALSE;

		// Must redraw.
		Invalidate();
	}
}

// v0.9.4.0
// Vert Scroll Bar showing?
BOOL CDataView::ScrollBarShowing()
{
	CRect client;
	GetClientRect(&client);
	CSize scrollSz = GetTotalSize();

	// If the vert bar is showing, height will be greater.
	return scrollSz.cy > client.Height();
}

/////////////////////////////////////////////////////////////////////////////
// CDataView drawing

void CDataView::OnDraw(CDC* pDC)
{
	CDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Set up font.
	if(!(HFONT)m_font)
	{
		LOGFONT lf = pDoc->m_viewLf;
		m_font.CreateFontIndirect(&lf);
		m_font.GetLogFont(&lf);
	}

	// If ListFiles showing, bail.
	if(pDoc->m_listFiles.ListShowing())
		return;

	// 10/04/2017
	// If no video playing, draw strings.
	// Note m_strAr set in FormatRawText().
	if(m_player.IsStopped())
	{
		// Draw into a back buffer.
		DrawBuffer(pDC);
	}
/*orig
	// If no video playing, draw strings.
	// Note m_strAr set in FormatRawText().
	if(m_player.IsStopped())
	{
		CFont * old = pDC->SelectObject(&m_font);
		CSize strSz = pDC->GetTextExtent(_T("X")); 

		// Format as needed.
		if(m_formatText)
		{
			// Update strings.
			FormatRawText(pDC);

			// Update sizes.
			SetScrollViewSize();

			// Reset. flag.
			m_formatText = FALSE;
		}

		// v0.9.4 - blank line added as first elemnt.
		//
		// Set first line 10 pix from left and zero down.
		CPoint pt(10, 0);

		// Draw the strings.
		int sz = m_strAr.GetSize();
		for(int i = 0; i < sz; i++)
		{
			pDC->TextOut(pt.x, pt.y, m_strAr[i]);
			pt.y += strSz.cy;
		}
		pDC->SelectObject(old);
	}*/

	// Playing
	//
	// Hide the video, repaint the View bkgnd, and show the
	// vid again. Needed when resizing or paused playback.
	//
	// See also CMainFrame::OnActivate(), OnSetActivate().
	// which are used when minimizing app.
	//
	// Flag set in OnSize() and OnEraseBkgnd().
	else if(m_redrawVid)
	{
		// Hide video.
		m_player.PutVisible(TRUE);	

		// Repaint bkgnd.
		CRect r;
		GetClientRect(&r);
		pDC->FillSolidRect(0, 0, 
						   r.Width(), r.Height(), 
						   m_vidBkgColor);//pDC->GetBkColor());
		// Show video.
		m_player.PutVisible(FALSE);

		// Reset.
		m_redrawVid = FALSE;
	}
}

// 10/04/2017
// Draw text into a back buffer to eliminate flicker during text selection.
// See also CDataView::OnEraseBkgnd().
void CDataView::DrawBuffer(CDC * pDC)
{
	CDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Get a logical clip box.
	CRect clip;
	pDC->GetClipBox(&clip);	// MM_TEXT

	// If no Clip - nothing to do.
	if(clip.IsRectEmpty())
		return;

	// Create and prepare a compatible memory DC.
	CDC memDC;
	memDC.CreateCompatibleDC(pDC);
	memDC.SetMapMode(MM_TEXT);		// default

	// Clean bitmap if it already exists.
	if(m_screenBitmap.m_hObject)
		m_screenBitmap.DeleteObject();

	// Create and select a clip box sized bitmap.
	m_screenBitmap.CreateCompatibleBitmap(pDC, 
										  clip.right  - clip.left,
										  clip.bottom - clip.top);	// MM_TEXT
	CBitmap * pOldBitmap = memDC.SelectObject(&m_screenBitmap);

	// Set the dc's window origin to the logical 
	// clip box's upper left.
	memDC.SetWindowOrg(clip.left, clip.top);

	// Paint with current background color.
	COLORREF bkgColor = pDC->GetBkColor();
	CBrush bkgBrush(bkgColor);
	CBrush * pOldBrush = memDC.SelectObject(&bkgBrush);
	memDC.PatBlt(clip.left,
				 clip.top,
				 clip.Width(),
				 clip.Height(),
				 PATCOPY);
	// Draw.
	CFont * old = memDC.SelectObject(&m_font);
	CSize strSz = memDC.GetTextExtent(_T("X")); 

	// Format strings as needed.
	if(m_formatText)
	{
		// Update strings.
		FormatRawText(&memDC);

		// Update sizes.
		SetScrollViewSize();

		// Reset. flag.
		m_formatText = FALSE;
	}

	// Set first line m_textOffset pix from top left client.
	CPoint pt(m_textOffset);

	// Draw the strings.
	int sz = m_strAr.GetSize();
	for(int i = 0; i < sz; i++)
	{
		memDC.TextOut(pt.x, pt.y, m_strAr[i]);
		pt.y += strSz.cy;
	}

	// If valid highlighted text.
	if(pDoc->m_curTabViewIndex == PAGE_CONTENTS && 
	   m_textSel.IsValid(m_strAr) )
	{
		// Draw the highlight strings.
		m_textSel.DrawHighlight(&memDC, pDoc->m_viewLf);
	}

	// Clean font.
	memDC.SelectObject(old);

	// Blast blit bitmap to the screen
	pDC->BitBlt(clip.left,    clip.top,
				clip.right  - clip.left,
				clip.bottom - clip.top, 
				&memDC,
				clip.left, clip.top, 
				SRCCOPY);
	
	// Clean brush.
	memDC.SelectObject(pOldBrush);
	// Select out bitmap
	memDC.SelectObject(pOldBitmap);
}

/////////////////////////////////////////////////////////////////////////////
// CDataView message handlers

void CDataView::OnSize(UINT nType, int cx, int cy) 
{
	CScrollView::OnSize(nType, cx, cy);
	// TODO: Add your message handler code here

	CDoc * pDoc = GetDocument();

	// If ListFiles is open, resize the list.
	if(pDoc->m_listFiles.ListShowing())
	{
		if(cx && cy)
		{
			CRect rect(0, 0, cx, cy);
			pDoc->m_listFiles.Resize(rect);
			return;
		}
	}

	// Update Video size.
	else if(m_player.IsRunningOrPaused() && cx && cy)
	{
		// Get client size.
		CRect rect;
		GetClientRect(&rect);

		// Set the size of the video window.
		if(m_player.SetVideoRect(rect))
		{
			// Set flag to redraw new video
			// size and View's background.
			m_redrawVid = TRUE;
		}
	}

	// Else format for new client size.
	else
	{
		// Set flags so raw text is reformatted and bars reset.
		// See OnDraw() and SetScrollViewSize().
		m_formatText = TRUE;		// v0.9.2.0
		m_scrollResizing = TRUE;	// v0.9.4.0
	}
}

// Override background painting when playing.
BOOL CDataView::OnEraseBkgnd(CDC* pDC) 
{
	// Redraw the video and bkgnd once.
	// See OnDraw().
	if(m_player.IsRunningOrPaused())
	{
		m_redrawVid = TRUE;
		return 1;			// 1 = handled
	}

	// For text display, we handle in Draw() and DrawBuffer().
	else if(!m_player.IsRunningOrPaused())
		return 1;

	// Else erase for video etc.
	else
		return CScrollView::OnEraseBkgnd(pDC);


	// TODO: Add your message handler code here and/or call default	
	return CScrollView::OnEraseBkgnd(pDC);
}
/* to do - verify text selection flicker for khands
// 10/04/2017
// Control background erase for Back Buffer drawing.
// See DrawBuffer().
BOOL CDataView::OnEraseBkgnd(CDC* pDC) 
{
	CDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// For text display, we handle in Draw() and DrawBuffer().
	if(!pDoc->m_vlcRunning)
		return TRUE;
	// Else erase for video etc.
	else
		return CScrollView::OnEraseBkgnd(pDC);
	
//	return CScrollView::OnEraseBkgnd(pDC);
}*/


void CDataView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	CDoc * pDoc = GetDocument();

	// Framework calling - bail
	if(lHint == FRAMEWORK_UPDATE) // 0
	{
		// Set window title
		GetParentFrame()->SetWindowText(_T("Khan Desktop"));
		return;
	}

	// Toggle List Files
	else if(lHint == TOGGLE_LIST_FILES)
	{
		if(m_player.IsRunningOrPaused())
			return;
		ToggleFilesList();
		return;
	}

	// Invalid rect.
	else if(lHint == INVALID_RECT)
	{
		// Should have a pHint CRect.
		if(pHint == NULL)
			return;
		CRect * pRect = (CRect*)pHint;
		InvalidateRect(pRect);
		UpdateWindow();
	}

	// New display strings set. Update View.
	// not used
	else if(lHint == DISPLAY_STRING)
	{
		// No redraw if playing or ListFiles open.
		if(!pDoc							|| 
		   m_player.IsRunningOrPaused()		||
		   pDoc->m_listFiles.ListShowing()	)
			return;

		// Redraw.
		Invalidate();
		UpdateWindow();
	}

	// Format and display Raw strings.
	// Called via CTabView::TabSelChange().
	else if(lHint == RAW_STRING)
	{
		// Format the new text.
		//FormatRawText();
		m_formatText = TRUE; //v0.9.2.0

		// No redraw if playing or ListFiles open.
		if(!pDoc							|| 
		   m_player.IsRunningOrPaused()		||
		   pDoc->m_listFiles.ListShowing()	)
			return;

		// 10/04/2017
		// Clean any selected text.
		m_textSel.Zero();

		// Update bars.
		SetScrollViewSize(); 

		// Redraw.
		Invalidate();
	}
	
	// Format and display Raw strings, and 
	// close the ListFiles if open.
	// Called via CTabView::TabSelChange().
	else if(lHint == RAW_STRING_EX)
	{
		// Format the new text.
		//FormatRawText();
		m_formatText = TRUE; //v0.9.2.0

		// No redraw if playing.
		if(!pDoc || m_player.IsRunningOrPaused())
			return;

		// If ListFiles is showing, close it..
		if(pDoc->m_listFiles.ListShowing())
			pDoc->m_listFiles.DestroyList();

		// 10/04/2017
		// Clean any selected text.
		m_textSel.Zero();

		// Update bars.
		SetScrollViewSize(); 

		// Redraw.
		Invalidate();
	}

	// Redraw.
	else if(lHint == REDRAW)
	{
		int g = 0;
	}
}

// Reworked 10/04/2017 search
// Format and word wrap the CDoc::m_rawStr raw text into strings 
// and save in the m_strAr array for View display.
// Called only from OnDraw().
void CDataView::FormatRawText(CDC * pDC)
{
	CDoc * pDoc = GetDocument();
	if(!pDoc)
	{
		m_strAr.RemoveAll();
		return;
	}
	CString str = pDoc->m_rawStr;
	if(str.IsEmpty())
	{
		m_strAr.RemoveAll();
		return;
	}

	// Set up font.
	if(!(HFONT)m_font)
	{
		LOGFONT lf = pDoc->m_viewLf;
		m_font.CreateFontIndirect(&lf);
	}

	// Clean existing.
	m_strAr.RemoveAll();

	// Tab - 4 spaces.
	CString tab =  _T("    ");
	int t, nTabs = 0;

	// Tab Indent
	BOOL wrapIndent = TRUE;

	CSize lineSz(0, 0);
	CSize wordSz(0, 0);
	CSize tabSz = pDC->GetTextExtent(tab);
	CString word, line;
	TCHAR c = _T('');
	TCHAR cNext = _T('');
	TCHAR cNextNext = _T('');
	TCHAR cPrev = _T('');
	TCHAR cPrevPrev = _T('');
	BOOL isEscape = FALSE;
	BOOL isNewline;

	CRect client;
	GetClientRect(&client);

	// If the vert bar is not showing, make room for it
	// in case it shows. If no show, text will wrap short
	// of right side, which is ok.
	int barWidth = 0;
	if(!ScrollBarShowing())
		barWidth = GetSystemMetrics(SM_CXVSCROLL);

	// Pad text 10 pix each side, plus vert bar width as needed. See OnDraw();
	int width = client.Width() - 20 - barWidth;

	// Add a blank line at the top for spacing.  v0.9.4
	m_strAr.Add(_T(""));

	// Format the raw text.
	int sz = str.GetLength();
	for(int i = 0; i < sz; i++)
	{
		// Get current, orevious and next chars.
		cPrevPrev = cPrev;
		cPrev = c;
		c = str[i];
		cNext = _T('');
		if(i + 1 < sz)
			cNext = str[i + 1];
		cNextNext = _T('');
		if(i + 2 < sz)
			cNextNext = str[i + 2];

		// Flags
		isNewline = FALSE;

		// Escape start - go around for char,
		if(c == _T('\\'))
			continue;

		// Handle escape chars.
		if(cPrev == _T('\\'))
		{
			// Newline n for second byte - ascii 110.
			if(c == _T('n'))
			{
				isNewline = TRUE;
			}

			// Backslash char.
			else if(c == _T('\\'))
			{ /*add below*/ }

			// Else not implemented yet - show the escape.
			else
			{ 
				word += _T('\\');
				wordSz = pDC->GetTextExtent(word);
			}
		}
		// Newline \n - ascii 10.
		else if(c == 10)
			isNewline = TRUE;

		// Tab \t - may be multiple.
		// Note traditional tabbing not implemented.
		// Tabs at start of line are used to indent 
		// wrapped text if wrapIndent is true.
		else if(c == 9)
		{
			// If a recent newline, zero count.
			// Each line sets its own tabs.
			if(cPrev == 10)
				nTabs = 0;
			word += tab;
			wordSz = pDC->GetTextExtent(word);

			// The total is used for this apps indenting.
			if(line.GetLength() == 0 &&
			   word.GetLength() == tab.GetLength() * (nTabs + 1) )
				nTabs ++;

			// Get next char - perhaps tab.
			continue;
		}

		// Add the char.
		if(!isNewline)
		{
			word += c;
			wordSz = pDC->GetTextExtent(word);
		}

		// Check wrap if a newline or space char.
		if(isNewline || c == _T(' '))
		{
			// dbg
			if(word == _T("Murasaki."))	// humanities Murasaki. 
				int g = 0;

			// Wrap word to next line.
			if(lineSz.cx + wordSz.cx > width)
			{
				m_strAr.Add(line);
				line.Empty();

				// If word wrapping tabs, add tabs to 
				// a new empty line.
				if(wrapIndent)
				{
					for(t = 0; t < nTabs; t++)
						line += tab;
				}
				// Add the word to the line.
				// If the word preceds a newline, the line
				// is added below.
				line += word;
				lineSz = pDC->GetTextExtent(line);
			}
			// Else add word to line.
			else
			{
				line += word;
				lineSz = pDC->GetTextExtent(line);
			}

			// If a newline after the word.
			if(isNewline)
			{
				// Add the string (may be an empty line).
				m_strAr.Add(line);
				line.Empty();
				lineSz = 0;

				// If a newline embedded in a string, indent the line if tabs.
				// Words will be appended to the line later.
				if(wrapIndent	&& // are wrapping
				   nTabs		&& // have tabs
				   cNext != 9	&& // tab not next
				   cNext != 10	&& // newline not next
				   cPrev != 10	&& // newline not before
				   i != sz -1	)  // not last char
				{
					for(t = 0; t < nTabs; t++)
						line += tab;
					lineSz = pDC->GetTextExtent(line);
				}
			}
			// Clean the word.
			word.Empty();
			wordSz.cx = 0;
		}

		// If the end, save any valid words/lines.
		if(i == sz - 1)
		{
			if(lineSz.cx + wordSz.cx >= width)
			{
				m_strAr.Add(line);
				line.Empty();
				// If word wrapping tabs.
				if(wrapIndent)
				{
					for(t = 0; t < nTabs; t++)
						line += tab;
				}
				line += word;
				m_strAr.Add(line);
			}
			else
			{
				line += word;
				m_strAr.Add(line);
			}
		}
	}
}

// out 5/15 fastly fixed
/*
/////////////////////////////////////////////////////////////////////////////
// Response Progress Ctrl Dialog - 12/13/14

void CDataView::StartResponseProgress()
{
	//
	// Here we run a dialog with a progress control
	// in Modal mode, blocking all other ui till
	// closed by CDoc::StreamTestAborted() or
	// CDoc::StreamTestFinished() via StopResponseProgress()
	// or by user pressing the cancel button.
	//

	// Start timer for stepping progress ctrl.
	m_progStep = 0;
	SetTimer(RESPONSE_TIMER, 500, NULL);

	int ret = m_progressDlg.DoModal();

	// User canceled the operation - dlg is now closed.
	if(ret == IDCANCEL)
	{
		// Kill timer.
		KillTimer(RESPONSE_TIMER);

		// Msg CDoc Thread that user has canceled opperation.
		CDoc * pDoc = GetDocument();
		if(pDoc)
			pDoc->StreamTestCancel();
	}
	// StopResponseProgress() was called - dlg is now closed.
	else //if(ret == 5)
	{
		// Kill timer.
		KillTimer(RESPONSE_TIMER);
	}
}
// Called from OnTimer() and CDoc::...
void CDataView::StopResponseProgress()
{
	// Close the dialog via DoModal().
	int ret = 5;
	if(m_progressDlg.GetSafeHwnd())
	{
		m_progressDlg.EndDialog(ret);
	}
}
*/

/////////////////////////////////////////////////////////////////////////////
// Video Controls Bar messages
//
// Controls Bar start button pressed.
//	Called from:
//	 CControlsBar::OnPlayPause()
//	 CTreeDialog::OnContextMenu() - PlayVideo item pressed.
LRESULT CDataView::OnPlaybackStart(WPARAM wParam, LPARAM lParam) // WM_USER_VID_START
{
	CDoc * pDoc = GetDocument();
	if(!pDoc)
		return 0;

	CString path;
	BOOL isUrl;

	// Get selected video.
	if(pDoc->GetCurrentVideoPath(path, isUrl))
	{
		// Play.
		StartVideoPlayback(path, isUrl);

		// Flag - not FileOpen.
		m_isKhanVideo = TRUE;
	}
	return 0;
}

// Controls Bar stop button pressed.
LRESULT CDataView::OnPlaybackStop(WPARAM wParam, LPARAM lParam)	// WM_USER_VID_STOP
{
	// Stop.
	StopVideoPlayback();

	// Focus off controls bar.
	SetFocus();
	return 0;
}

// Controls Bar pause button pressed - toggle.
LRESULT CDataView::OnPlaybackPause(WPARAM wParam, LPARAM lParam) // WM_USER_VID_PAUSE
{
	ToggleVideoPlaybackPause();
	return 0;
}

// Controls Bar mute button pressed - toggle.
LRESULT CDataView::OnPlaybackMute(WPARAM wParam, LPARAM lParam)	// WM_USER_VID_MUTE
{
	m_player.ToggleVolumeMute();	
	SetFocus();
	return 0;
}

// Controls Bar video slider changed.
// Called from CControlsBar OnHScroll() and OnReleasedcaptureSliderVid().
LRESULT CDataView::OnPlaybackTime(WPARAM wParam, LPARAM lParam)	// WM_USER_VID_TIME
{

	// Ensure.
	if(m_player.IsStopped())
		return 0;

	BOOL moving = wParam ? TRUE : FALSE;
	UINT newPos = (UINT)lParam; // 0 - 1000, 
	UINT seconds;

	// Update video time, and get the new position in seconds.
	if(m_player.PlaybackTimeSliderMoved(moving, newPos, seconds))
	{
		// User moving the slider.
		if(moving)
		{
			// Update the controls bar position time string. Note the lParam
			// newPos value is not used as user is moving the slider.
			//
			// Calls CControlsBar::OnVidTimeChange(). 
			HWND hwndBar = ((CApp*)AfxGetApp())->m_hwndControlBar;
			if(hwndBar)
				::PostMessage(hwndBar, WM_USER_BAR_TIME, (WPARAM)seconds, (LPARAM)newPos);
		}
		// Finished moving the slider - set focus to the View.
		else
			SetFocus();
	}

	/* dbg mesage
	CDsLavDoc* pDoc = GetDocument();
	CString str;
	str.Format(_T("Seek Pos %d"), (int)lParam);
	pDoc->m_statBarMsg = str;*/
	return 0;
}

// Controls Bar vol slider changed.
LRESULT CDataView::OnPlaybackVol(WPARAM wParam, LPARAM lParam)	// WM_USER_VID_VOL
{
	int percent = 0;

	// If paused, force the Controls Bar slider back to its
	// current value, effectively disabling the slider.
	if(m_player.IsPaused())
	{
		percent = m_player.GetVolPercent();

		// Set bar vol.
		HWND hwndBar = ((CApp*)AfxGetApp())->m_hwndControlBar;
		if(hwndBar)
			::PostMessage(hwndBar, WM_USER_BAR_VOL, 0, (LPARAM)percent);
		return 0;
	}

	// Else validate range.
	percent = (int)lParam;
	if(percent > 100)
		percent = 100;
	if(percent < 0)
		percent = 0;

	// Set new volume.
	if(m_player.PlaybackVolSliderMoved(percent))
	{
		/* dbg msg
 		CDoc * pDoc = GetDocument();
		pDoc->m_statBarMsg.Format(_T("Volume percent = %d"), percent);*/
	}
	SetFocus();

	return 0;
}

// Video Playback event callback from CPlayer.
LRESULT CDataView::OnVideoEvent(WPARAM wParam, LPARAM lParam)		// WM_USER_VIDEOEVENT
{
	// Get the event from CPlayer.
	LONG code;
	m_player.GetVideoEvent(code);

	// Playback has finished.
	if(code == EC_COMPLETE || code == EC_USERABORT)
	{
		TRACE(_T("In CDataView::OnVideoEvent Stopping Playback\n"));

		// Stop the playback.
		StopVideoPlayback();
	}
	return NOERROR;
}

// Called via message from CMainFrame::OnActivate().
LRESULT CDataView::OnSetActivate(WPARAM wParam, LPARAM lParam)	// WM_USER_SET_ACTIVATE
{
	// If deactivating, pause the video.
	if(lParam && m_player.IsRunning())
	{
		TRACE(_T("CDataView::OnSetActivate - de-activating"));
		ToggleVideoPlaybackPause();
	}
	// If activating, we leave the video paused.
	// OnDraw will repaint.
	else if(wParam && m_player.IsPaused())
	{
		TRACE(_T("CDataView::OnSetActivate - activating"));
	}
	return 0;
}

// Video playback is starting - set up the Controls Bar.
void CDataView::ControlsBarEnable()
{
	// Must be running.
	if(!m_player.IsRunning())
		return;

	// Get video duration.
	REFERENCE_TIME	totalTime = m_player.GetTotalTime();
	if(!totalTime)
		return;

	// Set a timer for ControlBar vid playback position. 
	// See OnTimer().
	// Update 30 times a second seconds.
	SetTimer(TIMER_VID_ID, 34, NULL);

	// Enable controls.
	HWND hwndBar = ((CApp*)AfxGetApp())->m_hwndControlBar;
	if(hwndBar)
		::PostMessage(hwndBar, WM_USER_BAR_START, 0, 0);
}

// Video playback is stopping - disable Controls Bar.
void CDataView::ControlsBarDisable()
{
	// Timer off.
	KillTimer(TIMER_VID_ID);

	// Disable controls.
	HWND hwndBar = ((CApp*)AfxGetApp())->m_hwndControlBar;
	if(hwndBar)
		::PostMessage(hwndBar, WM_USER_BAR_STOP, 0, 0);
}

// Controls Bar Vid playback time position handling.
// See ControlsBarEnable().
void CDataView::OnTimer(UINT nIDEvent) 
{
	// If playback not stopped and not paused.
	if(m_player.IsRunning())
	{
		UINT pos, seconds;
		if(m_player.PlaybackGetCurTime(pos, seconds))
		{
			// Update Controls bar.
			HWND hwndBar = ((CApp*)AfxGetApp())->m_hwndControlBar;
			if(hwndBar)
				::PostMessage(hwndBar, WM_USER_BAR_TIME, (WPARAM)seconds, (LPARAM)pos);

			// WPARAM is UINT: 4,294,967,295 / 60 / 60 = 1190 hrs 
		}
	}
	/* out 5/15 fastly fixed
	// Response timer 12/13/14
	else if(nIDEvent == RESPONSE_TIMER)
	{
		// Step the progress.
		if(m_progressDlg.GetSafeHwnd() )
		{
			m_progressDlg.StepProgressCtrl();

			// Close progress ctrl:
			//
			// Go for 30 seconds - 60 steps @ 500 ms each.
			// See: CKnanInet::TestInetStream2 and
			//		CProgressDlg::OnInitDialog().
			m_progStep ++;
			if(m_progStep > 60)
				StopResponseProgress();
		}
	}*/

	// Base	
	CScrollView::OnTimer(nIDEvent);
}

// Toggle video playback Pause / Play and
// update the Controls Bar.
void CDataView::ToggleVideoPlaybackPause()
{
	// Note CDoc flag m_videoPlaying flag 
	// remains TRUE while paused.

	// Toggle playback.
	m_player.TogglePlaybackPause();

	// Update Controls Bar.
	//
	// Paused.
	if(m_player.IsPaused())
	{
		// Set bar text to "Play" (depause).
		HWND hwndBar = ((CApp*)AfxGetApp())->m_hwndControlBar;
		if(hwndBar)
			::PostMessage(hwndBar, WM_USER_BAR_PAUSE, 0, 1);
	}
	else
	{
		// Set bar text to "Pause".
		HWND hwndBar = ((CApp*)AfxGetApp())->m_hwndControlBar;
		if(hwndBar)
			::PostMessage(hwndBar, WM_USER_BAR_PAUSE, 0, 0);
	}
	SetFocus();
}

// Stop playback
void CDataView::StopVideoPlayback()
{
	TRACE(_T("In CDataView StopVideoPlayback\n"));

	CDoc * pDoc = GetDocument();

	// Add a History item.
	if(m_isKhanVideo && pDoc)
	{
		UINT pos, seconds;
		if(m_player.PlaybackGetCurTime(pos, seconds))
		{
			pDoc->SetHistoryItem(seconds);

			// Update strings to show any new history.
			pDoc->SetDisplayStringsVideo(FALSE);
			m_formatText = TRUE;
		}
	}

	// Stop the playback.
	m_player.StopPlayback();

	// Set CDoc flag.
	if(pDoc)
		pDoc->m_videoPlaying = FALSE;

	// Redraw needed if playback was paused.
	Invalidate();
	UpdateWindow();

	// Disable bar.
	ControlsBarDisable();
}

// Play a video.
//	Called from: OnPlaybackStart(), OnFileOpen().
void CDataView::StartVideoPlayback(const CString & path, BOOL isUrl) 
{
	TRACE(_T("In CDataView StartVideoPlayback\n"));

	CString path2 = path;

	// Ensure a valid internet connection and url.
	// LAV provides poor error msg.
	if(isUrl)
	{
		TRACE(_T("In CDataView StartVideoPlayback - start Inet stream test\n"));
		CString errMsg;
		CKnanInet inet;
		if(!inet.TestInetStream(path2, errMsg))
		{
			MessageBox(errMsg, _T("Khan Desktop Application"), MB_OK | MB_ICONINFORMATION);
			return;
		}
	}

	// Hide Bars
	CSize sz(0, 0);
	SetScrollSizes(MM_TEXT, sz); // zero sz ok

	// Redraw
	Invalidate();
	UpdateWindow();

	// Get new client area.
	CRect client;
	GetClientRect(&client);

	// Clean text.
	CClientDC dc(this);
	dc.FillSolidRect(0, 0, 
					 client.Width(), client.Height(), 
					 m_vidBkgColor);//dc.GetBkColor());


	TRACE(_T("In CDataView StartVideoPlayback - calling PlayVideoLav()\n"));

	// dbg
//	DWORD start = GetTickCount();

	// Play from disk or URL.
	if(m_player.PlayVideoLav(path2, m_hWnd, client))
	{
		TRACE(_T("In CDataView StartVideoPlayback - PlayVideoLav() returned ok.\n"));

		// Set controls bar.
		ControlsBarEnable();

		// Set CDoc flag.
		CDoc * pDoc = GetDocument();
		pDoc->m_videoPlaying = TRUE;
	}
	// Error.
	else
	{
		SetScrollViewSize();
		Invalidate();
		UpdateWindow();
	}

	// dbg - Fastly stream - 200 ms for disk, 813 ms for http, 20422 ms for non cached http
//	TRACE(_T("In CDataView StartVideoPlayback - delay = %d.\n"), GetTickCount() - start);

	////////////////
	/* 11/04/2017 - out old fastly - above copied from VS2005
	// 12/13/14
	// If a URL, ensure a valid internet connection and url.
	// LAV provides poor error msg.
	// Call CDoc::StreamCurrentVideo() which will verify stream
	// and then call CDataView::PlayVideoStream().
	if(isUrl)
	{
		CDoc * pDoc = GetDocument();
		if(!pDoc)
			return;

		// Start CDoc thread.
		if(pDoc->StreamCurrentVideo())
			StartResponseProgress();

		return;
	}

	// Hide Bars
	CSize sz(0, 0);
	SetScrollSizes(MM_TEXT, sz); // zero sz ok

	// Get client area.
	CRect client;
	GetClientRect(&client);

	// Clean text.
	CClientDC dc(this);
	dc.FillSolidRect(0, 0, 
					 client.Width(), client.Height(), 
					 m_vidBkgColor);//dc.GetBkColor());

	// Play.
	if(m_player.PlayVideoLav(path, m_hWnd, client))
	{
		// Set controls bar.
		ControlsBarEnable();

		// Set CDoc flag.
		CDoc * pDoc = GetDocument();
		pDoc->m_videoPlaying = TRUE;
	}
	// Error.
	else
	{
		SetScrollViewSize();
		Invalidate();
		UpdateWindow();
	}*/
}

/*out fastly fixed
// 12/13/14
// Play a video stream.
//	Called from CDoc::StreamTestFinished() only.
void CDataView::PlayVideoStream(const CString & path) 
{
	// Hide Bars
	CSize sz(0, 0);
	SetScrollSizes(MM_TEXT, sz); // zero sz ok

	// Get client area.
	CRect client;
	GetClientRect(&client);

	// Clean text.
	CClientDC dc(this);
	dc.FillSolidRect(0, 0, 
					 client.Width(), client.Height(), 
					 m_vidBkgColor);//dc.GetBkColor());

	// Play.
	if(m_player.PlayVideoLav(path, m_hWnd, client))
	{
		// Set controls bar.
		ControlsBarEnable();

		// Set CDoc flag.
		CDoc * pDoc = GetDocument();
		pDoc->m_videoPlaying = TRUE;
	}
	// Error.
	else
	{
		SetScrollViewSize();
		Invalidate();
		UpdateWindow();
	}
}*/

////////////////////////////////////////////////////////////////////
// Message handlers

// File Open menu item selected.
// Open a non khan video.
void CDataView::OnFileOpen() //ID_FILE_OPEN
{
	TCHAR filters[] = _T("Media Files (*.avi*.mp4*.flv*.wmv)|*.avi;*.mp4;*.flv;*.wmv|All Files (*.*)|*.*||");   
  
	CFileDialog dlg(TRUE, NULL, NULL, OFN_NOCHANGEDIR   | 
									  OFN_FILEMUSTEXIST | 
									  OFN_PATHMUSTEXIST, 
									  filters);
	// Proccess selection
	if(dlg.DoModal() == IDOK)
	{
		// Set flag.
		m_isKhanVideo = FALSE;

		CRect client;
		GetClientRect(&client);
		CString filePath = dlg.GetPathName();

		// Play disk file.
		StartVideoPlayback(filePath, FALSE);
	}	
}
// Update
void CDataView::OnUpdateFileOpen(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	if(m_player.IsStopped())
		pCmdUI->Enable(TRUE);	
	else
		pCmdUI->Enable(FALSE);
}

// Toggle Video pause with space bar.
void CDataView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// Space bar down.
	//
	// Pause / depause a playing video.  
	if(nChar == VK_SPACE && ((nFlags & KF_REPEAT) == 0) )  
	{
		if(m_player.IsRunningOrPaused())
		{
			// Toggle Paused.
			ToggleVideoPlaybackPause();
		}
	}
	CScrollView::OnKeyDown(nChar, nRepCnt, nFlags);
}

// 10/04/2017
// Text selection
void CDataView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CDoc* pDoc = GetDocument();

	// If no ListFiles or Video.
	if(!pDoc->m_listFiles.ListShowing() &&
	   !m_player.IsRunningOrPaused()	)
	{
		// Set start pos for text sel - log units for scroling.
		if(pDoc->m_curTabViewIndex == PAGE_CONTENTS)
		{
			CClientDC dc(this);
			OnPrepareDC(&dc);
			CPoint p = point;
			dc.DPtoLP(&p);
			CRect clip;
			clip.SetRectEmpty();

			// If a selection has been made, user can shift-click
			// elswhere in the text to select a batch of text.
			// Similar to notepad.
			BOOL shiftDown = GetKeyState(VK_SHIFT);
			if(shiftDown & 0x8000)
			{
				// Same as OnMouseMove();
				if(m_textSel.ValidEndPt(p))
				{
					// Must select font into dc for SetEndDragPt().
					dc.SelectObject(m_font);
					// Set chars.
					if(m_textSel.SetEndDragPt(p, m_strAr, dc, m_textOffset, clip) ||
					   !clip.IsRectEmpty()										  )
					{
						clip.InflateRect(2, 2);
						dc.LPtoDP(&clip);
						InvalidateRect(clip);
						UpdateWindow();
					}
				}
			}
			// Else beggining a drag
			else
			{
				m_textSel.SetStartDragPt(p, clip);
				// Redraw any current selection as needed.
				if(!clip.IsRectNull())
				{
					clip.InflateRect(2, 2);
					dc.LPtoDP(&clip);
					InvalidateRect(clip);
					UpdateWindow();
				}
			}
		}
	}	
	CScrollView::OnLButtonDown(nFlags, point);
}

// 10/04/2017
// Context menu for copying selected text.
void CDataView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	// Must be clicking on selected text
	CClientDC dc(this);
	OnPrepareDC(&dc);
	CPoint p = point;
	dc.DPtoLP(&p);
	if(!m_textSel.PointInSelection(p))
		return;

	TRACE(_T("Entering CDataView OnContextMenu\n"));

	int index = 0;
	CString str;

	// Create the context.
	CMenu menu;
	menu.CreatePopupMenu();

	// Add single menu item - copy text.
	str = _T("Copy");
	menu.AppendMenu(MF_STRING, ID_CTX_COPY_TEXT, str);

	// Ensure.
	if(menu.GetMenuItemCount() == 0)
		return;

	// Need screen coords for menu pos.
	ClientToScreen(&point);

	// Show Context Menu.
	int nCmd = menu.TrackPopupMenu(	TPM_LEFTALIGN  | 
									TPM_LEFTBUTTON |
									TPM_RETURNCMD,		// see sdk TrackPopupMenu
									point.x, point.y, 
									this);
	// Copy selected text.
	if(nCmd == ID_CTX_COPY_TEXT)
		OnEditCopy();
	
	// Base.
	CScrollView::OnRButtonDown(nFlags, point);
}

// 10/04/2017
// Get text highlight.
void CDataView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CDoc* pDoc = GetDocument();

	// If no ListFiles or Video.
	if(!pDoc->m_listFiles.ListShowing() &&
	   !m_player.IsRunningOrPaused()	&&
	   (nFlags && MK_LBUTTON)			)
	{
		// Bail if shift down. See OnLButtonDown().
		BOOL shiftDown = GetKeyState(VK_SHIFT);
		if(shiftDown & 0x8000)
		{
			CScrollView::OnMouseMove(nFlags, point);
			return;
		}

		// Set highlight chars.
		if(pDoc->m_curTabViewIndex == PAGE_CONTENTS)
		{
			CClientDC dc(this);
			OnPrepareDC(&dc);
			CPoint p = point;
			dc.DPtoLP(&p);
			CRect clip;
			clip.SetRectEmpty();
			if(m_textSel.ValidEndPt(p))
			{
				// Must select font into dc for SetEndDragPt().
				dc.SelectObject(m_font);
				// Set chars.
				if(m_textSel.SetEndDragPt(p, m_strAr, dc, m_textOffset, clip) ||
				   !clip.IsRectEmpty()										  )
				{
					clip.InflateRect(2, 2);
					dc.LPtoDP(&clip);
					InvalidateRect(clip);
					UpdateWindow();
				}
			}
		}
	}	
	CScrollView::OnMouseMove(nFlags, point);
}

// 10/04/2017
// Get selected text into clipboard. ID_EDIT_COPY
void CDataView::OnEditCopy() 
{
	CString str;

	// If text available.
	if(m_textSel.GetSelectedText(str))
	{
		TRACE(_T("Entering CDataView::OnEditCopy\n"));

		// If we can get the clipboard.
		if(OpenClipboard())
		{
			// Clean up.
			EmptyClipboard();

			// Allocate - plus one for null teminator.
			int len = str.GetLength();
			HANDLE hData = GlobalAlloc(GMEM_MOVEABLE, sizeof(TCHAR) * (len + 1));
			if(!hData)
			{
				CloseClipboard();
				return;
			}

			// Get a ptr to clipboard mem.
			LPTSTR pData = (LPTSTR) GlobalLock(hData);

			// Copy to clipboard.
			//
			// Note GetBuffer() returns an LPTSTR pointer to the
			// strings (null-terminated) character buffer.
			//
			// Also, the length param is the minimum size of the 
			// character buffer in characters. This value does 
			// not include space for a null terminator.
			//
			// See msdn GetBuffer().
			//
			StringCbCopy(pData,									// ok VC6 KhanDS
						 (str.GetLength() + 1) * sizeof(TCHAR), 
						 str);
			//lstrcpy(pData, str.GetBuffer(len));				// ok VC6 KhanVlc, not KhanDS

			// Release str buf.
			str.ReleaseBuffer();

			// Pass control of the mem to the clipboard.
			GlobalUnlock(hData);

			// Set the clipboard.
			//
			#ifdef UNICODE
				SetClipboardData(CF_UNICODETEXT, hData);
			#else
				SetClipboardData(CF_TEXT, hData);
			#endif

			// Close.
			CloseClipboard();
			
			return;
		}
		// Clipboard not available.
		TRACE(_T("OnEditCopy - clipboard not available.\n"));

		return;	
	}	
}

// Mouse wheel rotation.
// v0.9.4.0 revised for View text scrolling.
BOOL CDataView::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
	CDoc * pDoc = GetDocument();

	BOOL stepUp = zDelta > 0;
	LONG newVol = 0;

	// If Video is playing.
	if(m_player.IsRunningOrPaused())
	{
		if(m_player.IsRunning() &&
		   m_player.StepVolume(stepUp, newVol))
		{
			// Set bar vol.
			HWND hwndBar = ((CApp*)AfxGetApp())->m_hwndControlBar;
			if(hwndBar)
				::PostMessage(hwndBar, WM_USER_BAR_VOL, 0, (LPARAM)newVol);
		}
	}

	// Else Scroll right pane Text.
	else if(GetFocus() == this)
	{
		// v0.9.4.0
		// Wheel scroll one line per msg.
		if(ScrollBarShowing())
		{
			if((HFONT)m_font)
			{
				CClientDC dc(this);
				dc.SelectObject(&m_font);
				CSize sz = dc.GetTextExtent(_T("X"));
				CPoint pos = GetScrollPosition();
				if(zDelta > 1)
					pos.y -= sz.cy;
				else
					pos.y += sz.cy;

				// Ensure an even line height. There will most likely be 
				// extra space at the bottom of the view which gets added
				// when scrolling to the the bottom and then returned by
				// GetScrollPosition() when scrolling back up.
				// Also, if user drags the scroll bar the spacing may be
				// set to any random pos.
				if(pos.y && pos.y % sz.cy != 0)
					pos.y -= pos.y % sz.cy;

				// Scroll.
				ScrollToPosition(pos);

				// Handled.
				return 0;
			}
		}
	}
	// Base class.
	return CScrollView::OnMouseWheel(nFlags, zDelta, pt);
}

// Set View font.
void CDataView::OnSetupFont() 
{
	CDoc * pDoc = GetDocument();

	if(pDoc)
	{
		CFontSetup dlg;
		dlg.SetCurFont(pDoc->m_viewLf);
		if(dlg.DoModal() == IDOK)
		{
			LOGFONT lf = dlg.GetLogfont();

			// Set up font.
			if((HFONT)m_font)
				m_font.DeleteObject();
			pDoc->m_viewLf = lf;
			m_font.CreateFontIndirect(&lf);

			// Redraw
			//FormatRawText();
			m_formatText = TRUE; //v0.9.2.0
			SetScrollViewSize();
			Invalidate();
		}
	}	
}

/////////////////////////////////////////////////////////////////
// List Files Tool

// Show / hide the list.
void CDataView::ToggleFilesList()
{
	CDoc * pDoc = GetDocument();
	if(!pDoc)
		return;

	// List showing.
	if(pDoc->m_listFiles.ListShowing())
	{
		// Clean list.
		pDoc->m_listFiles.DestroyList();

		// Reset strings.
		//FormatRawText();
		m_formatText = TRUE; //v0.9.2.0
		SetScrollViewSize();
		Invalidate();
	}
	// No list.
	else
	{
		// Clean view of text and scroll bars.
		m_strAr.RemoveAll();
		SetScrollViewSize();
		Invalidate();
		CRect client;
		GetClientRect(&client);
		UpdateWindow();

		// Show the list.
		pDoc->m_listFiles.CreateList(this, client);

		// Show the Contents tab and highlight any 
		// current video tree item.
		if(pDoc->m_listFiles.ListShowing())
		{
			HWND hWnd = ((CApp *)AfxGetApp())->m_hwndTabView;
			if(hWnd)
				::PostMessage(hWnd, WM_USER_PAGE_SWITCH, 0, PAGE_CONTENTS); // Contents Tab
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
// CDataView diagnostics

#ifdef _DEBUG
void CDataView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CDataView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CDoc* CDataView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CDoc)));
	return (CDoc*)m_pDocument;
}
#endif //_DEBUG


////////////////////////////////////////////////////////////
// testing

// "Test" menu pressed.
// Various debug tests - save,
// Set a temorary menu item to ID_TEST to use this code.
void CDataView::OnTest() 
{
	return;
	//////////////////////////////////////////////////////////////
	// Use the "test" versions of CKhanJson for debugging - save:
	//
	// First in debug mode, run the Khan Update feature. That will 
	// save a copy of the downloaded JSON data to C:\Khan.dat.
	// See CDoc::UpdateFinished() and CDoc::UpdateAborted().
	//
	// With that file copied, call here anytime via a test button 
	// with an id of ID_TEST.
	//
	//
	/////////////////////////////////////////////
	// Test new json parser
	CParseJson parse;
	CTopicTree topicTree;
	CString msg;
	BOOL ret = FALSE;

	// Run the parse code on C:\Khan.dat, saving results in temp tree.
	ret = parse.TestParseLibrary(&topicTree, _T("C:\\Khan.dat"), msg);
	// test prod code:
//	int progPos;
//	ret = parse.ParseLibrary(&topicTree, _T("C:\\Khan.dat"), NULL, progPos, msg);

	// Sort temp tree.
	BOOL bRoot = TRUE;
	BOOL bChild = FALSE;
	BOOL bVideo = FALSE; //FALSE TRUE
	topicTree.Sort(bRoot, bChild, bVideo);

	// Print Topics and vids to "C:\\KhanLib.txt".
	// Creates a text file of sorted Topics and their vids.
	topicTree.PrintTree(_T("C:\\KhanLib.txt"));

	// Format and print a somewhat readable version of the raw downloaded
	// json data now stored in C:\\Khan.dat. Good for visual debuging.
	// It simply adds newlines and tabs.
	parse.TestUtilityFormat(_T("C:\\Khan.dat"), _T("C:\\format.txt"));

	// Print raw slugs gotten while parsing.
	parse.TestUtilityPrintSlugs(_T("C:\\slugs.txt"), FALSE);// no sort

	// Serialize to program folder. This code was used for the 
	// very first creation of KhanDB.dat, and has no use now.
	// KhanDB.dat will be used during start up, and should be 
	// backed up first in case tree.Write() corrupts it. 
	//parse.Write(); // caution!

	// Message
	if(ret)
	{
		int numTopics = topicTree.NumTopics();			// 1923	4/18/2015
		int nodeBraceErr = parse.m_nNodeBraceError;		// 0
		int braceMissmatch = parse.m_nBraceMismatch;	// 194
		msg.Format(_T("Test: %d Topics gotten. %d node brace errors. %d brace mismatch errors"), 
			numTopics, nodeBraceErr, braceMissmatch);
		TRACE(_T("Test: %d Topics gotten. %d node brace errors. %d brace mismatch errors\n"), 
			numTopics, nodeBraceErr, braceMissmatch);
	}
	MessageBox(msg);
	return;
}




