
/*	Copyright (C) G. Jespersen
	LAV Filters - Copyright (C) 2010-2013 Hendrik Leppkes http://www.1f0.de
	See LAV Licence.txt.*/

//
// 06/07/16 - Use LAV 0.68.1
//

#if !defined PLAYER_H
#define PLAYER_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// Video Playback.
#include <dshow.h>
#include <comdef.h> // IUnknownPtr

#pragma comment(lib,"Strmiids.lib") // options: vc98 libs before sdk - vc6
#pragma comment(lib,"Quartz.lib")	// AMGetErrorText()


// For CreateObjectFromPath().
// Based on GCDL - http://www.gdcl.co.uk/2011/June/UnregisteredFilters.htm
typedef HRESULT (__stdcall * DllGetBaseFilterFunc)(REFCLSID, REFIID, void**);

///////////////////
// Manifest Notes:
// Add .ax files to manifest - no - won't run - probably bad path or format?
// Note: These are loaded via AfxLoadLibrary() (same as LoadLibrary) in
//		 CPlayer::CreateObjectFromPath().
// And finally - don't manifest these dll's - just let the app load them from paths.
//
// So in Khan Properties, Manifet Tool, Input and Output, Additional Manifetst Files:
//	Khan.exe.manifest;LAV\LAVFilters.Dependencies.manifest
//		Khan,exe,manifest sets dpi etc -
//		LAV\LAVFilters.Dependencies.manifest shows lave files -
//
//#pragma comment(linker, "\"/manifestdependency:type='Win32' name='LAVAudio_1.ax'\"")
//#pragma comment(linker, "\"/manifestdependency:type='Win32' name='LAVSplitter_1.ax'\"")
//#pragma comment(linker, "\"/manifestdependency:type='Win32' name='LAVVideo_1.ax'\"")


////////////////////////////////////////////////////////////////
// class CPlayer
//
// Note Com is initialized in CApp::InitInstance()
//

class CPlayer
{
public:
	CPlayer();
	~CPlayer();

private:
	// Video playback.
	IGraphBuilder * m_pGraph;
	IMediaControl * m_pControl;
	IMediaEventEx * m_pEvent;
	IMediaSeeking * m_pSeeking;
	IVideoWindow  * m_pVidWin;
	IBasicVideo	  * m_pBaseVid;
	IBasicAudio	  * m_pBaseAudio;
	BOOL			m_redrawVid;
	BOOL			m_muted;
	int				m_curVolPercent;	// 0 - 100
	HWND			m_hWnd;
	int				m_nPlays;			// debug

	// LAV Filter pointers.
	IBaseFilter	  *	m_pLavSrc;
	IBaseFilter	  *	m_pLavVid;
	IBaseFilter	  *	m_pLavAud;

	// Lav dll paths
	CString			m_pathLavSrc;
	CString			m_pathLavVid;
	CString			m_pathLavAud;

	// Misc
	CString			m_curDirectory;
	FILTER_STATE	m_fsCurrent;
	REFERENCE_TIME  m_rtTotalTime;

// Operations
public:

	// From View - ControlsBar changes.
	BOOL	PlaybackTimeSliderMoved(BOOL moving, const UINT & newPos, UINT & seconds);
	BOOL	PlaybackVolSliderMoved(int newPos);
	BOOL	PlaybackGetCurTime(UINT & position, UINT & seconds);	

	// From View - start, stop, size, etc.
	BOOL	PlayVideoLav(const CString & path, HWND hwnd, CRect rect);
	BOOL	StepVolume(BOOL stepUp, LONG & newVol);
	BOOL	TogglePlaybackPause();
	BOOL	GetVideoEvent(LONG & evCode);
	BOOL	SetVideoRect(CRect rect);
	void	ToggleVolumeMute();
	void	StopPlayback();
	void	PausePlayback();
	void	PutVisible(BOOL hide);

	// Get
	BOOL	IsPaused() { return m_fsCurrent == State_Paused; };
	BOOL	IsRunning() { return m_fsCurrent == State_Running; };
	BOOL	IsStopped() { return m_fsCurrent == State_Stopped; };
	BOOL	IsRunningOrPaused() { return !IsStopped(); };
	int		GetVolPercent() { return m_curVolPercent; };
	REFERENCE_TIME	GetTotalTime() { return m_rtTotalTime; };

private:

	// Helpers
	BOOL	CleanPlayback();
	BOOL	CleanPlayback(const CString & errMsg);
	BOOL	CleanPlayback(const CString & errMsg, int hr);
	void	PrepareSeeking();
	int		DxVolToPercent(LONG dxVol);
	LONG	DxVolFromPercent(int percent);

	// Use IBaseFilter pointers for LAV Filter creation. See PlayVideoLav().
	HRESULT CreateObjectFromPath(CString path, REFCLSID clsid, IBaseFilter ** pBaseFilter);

	// Pin fetch.
	HRESULT GetPin(IBaseFilter *pF, PIN_DIRECTION PinDir, 
				   const GUID& Category, IPin **ppPin);
	BOOL	PinMatchesCategory(IPin *pPin, const GUID & Category, PIN_DIRECTION PinDir);
	void	UtilDeleteMediaType(AM_MEDIA_TYPE *pmt);

	// Debug
	HRESULT EnumFilters();
};

#endif // _PLAYER_H
