
/*	Copyright (C) G. Jespersen
	See Licence.txt.*/

#include "Stdafx.h"
#include "Khan.h"
#include "TextSel.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// 10/04/2017
//////////////////////////////////////////////////////////////
// CStrHglt - a single string to highlight
CStrHglt::CStrHglt()
{
	Zero();
}
CStrHglt::~CStrHglt()
{
}
CStrHglt::CStrHglt(const CStrHglt & strHglt)
{
	Copy(strHglt);
}

void CStrHglt::operator=(const CStrHglt & strHglt)
{
	Copy(strHglt);
}

BOOL CStrHglt::operator==(const CStrHglt & strHglt)
{
	if(m_str		 == strHglt.m_str			&&
	   m_rect		 == strHglt.m_rect			&&
	   m_srcArElmt	 == strHglt.m_srcArElmt		&&
	   m_iStartChar	 == strHglt.m_iStartChar	&&
	   m_iEndChar	 == strHglt.m_iEndChar		&&
	   m_isEmptyLine == strHglt.m_isEmptyLine	)
		return TRUE;
	else
		return FALSE;
}

BOOL CStrHglt::operator!=(const CStrHglt & strHglt)
{
	return !(*this == strHglt);
}

void CStrHglt::Copy(const CStrHglt & strHglt)
{
	m_str = strHglt.m_str;
	m_rect = strHglt.m_rect;
	m_iStartChar = strHglt.m_iStartChar;
	m_iEndChar = strHglt.m_iEndChar;
	m_srcArElmt = strHglt.m_srcArElmt;
	m_isEmptyLine = strHglt.m_isEmptyLine;
}

void CStrHglt::Zero()
{
	m_str = _T("");
	m_rect.SetRectEmpty();
	m_iStartChar = -1;
	m_iEndChar = -1;
	m_srcArElmt = -1;
	m_isEmptyLine = FALSE;
}



/////////////////////////////////////////////

CTextSel::CTextSel()
{
	Zero();
}
CTextSel::~CTextSel()
{
}

CTextSel::CTextSel(const CTextSel & textSel)
{
	Copy(textSel);
}

void CTextSel::operator=(const CTextSel & textSel)
{
	Copy(textSel);
}

BOOL CTextSel::operator==(const CTextSel & textSel)
{
	// Check array
	int sz = m_strHgltAr.GetSize();
	int szSrc = textSel.m_strHgltAr.GetSize();

	// Sizes match.
	if(sz != szSrc)
		return FALSE;

	// Items match.
	for(int i = 0; i < sz; i++)
	{
		if(m_strHgltAr[i] != textSel.m_strHgltAr[i])
			return FALSE;
	}
	// ok.
	return TRUE;
}

BOOL CTextSel::operator!=(const CTextSel & textSel)
{
	return !(*this == textSel);
}

void CTextSel::Copy(const CTextSel & textSel)
{
	m_strHgltAr.Copy(textSel.m_strHgltAr);
}
void CTextSel::Zero()
{
	PointZero(m_dragStartPt);
	PointZero(m_dragEndPt);
	m_strHgltAr.RemoveAll();
}

// Check if stored highlight data is valid 
// for the current text display.
BOOL CTextSel::IsValid(const CStringArray & srcStrAr)
{
	CString hgltStr, srcStr;
	int srcArElmt, hgltStartChar;

	// Must have highlight strings.
	int szHgltAr = m_strHgltAr.GetSize();
	if(szHgltAr == 0)
		return FALSE;

	//dbg
	if(szHgltAr == 2)
		int g = 0;


	// Get source size.
	int szSrcAr = srcStrAr.GetSize();

	// Ensure highlite strings match source.
	for(int i = 0; i < szHgltAr; i++)
	{
		// Check source array bounds.
		srcArElmt = m_strHgltAr[i].m_srcArElmt;
		if(srcArElmt == -1 || srcArElmt >= szSrcAr)
			return FALSE;

		// Get strings
		hgltStr = m_strHgltAr[i].m_str;
		srcStr = srcStrAr[srcArElmt];

		// If an empty highligt str, nothing to check.
		if(m_strHgltAr[i].m_isEmptyLine)
			continue;

		// Otherwise, each should have chars.
		if(hgltStr.IsEmpty() || srcStr.IsEmpty())
			return FALSE;

		// Get the source string element where 
		// the highlight string begins.
		// Ensure element bounds.
		hgltStartChar = m_strHgltAr[i].m_iStartChar;
		if(hgltStartChar < 0 || hgltStartChar >= srcStr.GetLength())
			return FALSE;

		// Check that the source string contains the highlight string
		// which begins at the m_iStartChar of the srcStr.
		if(srcStr.Find(hgltStr, hgltStartChar) == -1)
			return FALSE;
	}
	// Ok
	return TRUE;
}

// Set hightlight strings.
// Note dc param must have View font object selected in.
BOOL CTextSel::SetHightlightStrings(const CStringArray & srcStrAr, CClientDC & dc,
									CSize textOffset, CRect & clipRect)
{
	// Ensure valid offset.
	if(textOffset.cx < 0 || textOffset.cy < 0)
		return FALSE;

	// Ensure valid start/end pts.
	if(PointIsEmpty(m_dragStartPt) || PointIsEmpty(m_dragEndPt) )
		return FALSE;

	// Need formatted strings.
	int strArSz = srcStrAr.GetSize();
	if(!strArSz)
		return FALSE;

	// Get current highlighting rect, if any.
	clipRect.SetRectEmpty();
	clipRect |= GetClipRect();

	// Prep string highlight ar.
	m_strHgltAr.RemoveAll();

	// Locals.
	int startLineElmt = -1;
	int endLineElmt = -1;
	CPoint dragStartPt = m_dragStartPt;
	CPoint dragEndPt = m_dragEndPt;

	// Get line height.
	CSize cs = dc.GetTextExtent(_T("X"));
	int lineHgt = cs.cy;

	// Get the start and end line elements from the pt 
	// members set in OnLButtonDown() and OnMouseMove.
	int vert = textOffset.cy;
	for(int i = 0; i < strArSz; i++)
	{
		// Start line.
		if(startLineElmt == -1)
		{
			if(dragStartPt.y >= vert &&				// need <= when crossing lines
			   dragStartPt.y  < vert + lineHgt)
				startLineElmt = i;
		}
		// End line.
		if(endLineElmt == -1)
		{
			if(dragEndPt.y >= vert && 
			   dragEndPt.y  < vert + lineHgt)
				endLineElmt = i;
		}
		vert += lineHgt;

		// Done.
		if(startLineElmt != -1 && endLineElmt != -1)
			break;
	}
	// Ensure.
	if(startLineElmt == -1 || endLineElmt == -1)
		return FALSE;

	// Reverse if user selecting multiple lines bottom up.
	if(startLineElmt > endLineElmt)
	{
		int i = startLineElmt;
		startLineElmt = endLineElmt;
		endLineElmt = i;
		CPoint p = dragStartPt;
		dragStartPt = dragEndPt;
		dragEndPt = p;
	}
	// Reverse if user selecting chars left to right on single line.
	else if(startLineElmt == endLineElmt)
	{
		if(dragStartPt.x > dragEndPt.x)
		{
			CPoint p = dragStartPt;
			dragStartPt = dragEndPt;
			dragEndPt = p;
		}
	}

	// Ensure again.
	if(startLineElmt >= strArSz || endLineElmt >= strArSz)
		return FALSE;

	// NOTE !!!
	// We are now organized as if a top-down and left-right drag.
	if(startLineElmt > endLineElmt)
		return FALSE;

	// Get the stings user is selecting
	CString startStr = srcStrAr[startLineElmt];
	CString endStr   = srcStrAr[endLineElmt];

	// Get the string's lengths.
	int startStrLen = startStr.GetLength();
	int endStrLen   = endStr.GetLength();

	// Get the string's extents.
	CSize startStrSz = dc.GetTextExtent(startStr);
	CSize endStrSz   = dc.GetTextExtent(endStr);

	// Get rectangles bounding start and end strings.
	CRect startStrRect;
	startStrRect.left	= textOffset.cx;
	startStrRect.top	= textOffset.cy + startLineElmt * lineHgt;
	startStrRect.right  = textOffset.cx + startStrSz.cx;
	startStrRect.bottom = startStrRect.top + lineHgt;

	CRect endStrRect;
	endStrRect.left	  = textOffset.cx;
	endStrRect.top	  = textOffset.cy + endLineElmt * lineHgt;
	endStrRect.right  = textOffset.cx + endStrSz.cx;
	endStrRect.bottom = endStrRect.top + lineHgt;

	/////////////////////////////
	// If highlight on one line.
	if(startLineElmt == endLineElmt)
	{
		CStrHglt strHglt;
		strHglt.m_str = startStr;
		strHglt.m_srcArElmt = startLineElmt;
		strHglt.m_rect = startStrRect;

		///////////////////////
		// If an empty string.
		if(startStrLen == 0)
		{
			strHglt.m_isEmptyLine = TRUE;
			// Done - add to array.
			m_strHgltAr.Add(strHglt);
			// Add to clip
			clipRect |= strHglt.m_rect;
			return TRUE;
		}
		//////////////////////////////
		// Else find start end chars.
		CRect rChar = startStrRect;
		CRect rStr  = startStrRect;
		CSize sz;
		for(int i = 0; i < startStrLen; i++)
		{
			sz = dc.GetTextExtent(startStr[i]);
			rChar.right = rChar.left + sz.cx;

			// Get highlight start.
			if(strHglt.m_iStartChar == -1)
			{
				if(rChar.PtInRect(dragStartPt)   ||		// on a char
				   dragStartPt.x < textOffset.cx )		// before string
				{
					strHglt.m_iStartChar = i;
					rStr.left = rChar.left;
				}
			}
			// Get highlight end.
			if(strHglt.m_iEndChar == -1)
			{
				if(rChar.PtInRect(dragEndPt))
				{
					strHglt.m_iEndChar = i;
					rStr.right = rChar.right;
				}
				else if(i == startStrLen - 1			 &&
						dragEndPt.x > startStrRect.right )
				{
					strHglt.m_iEndChar = i;
					rStr.right = rChar.right;
				}
			}
			// If done
			if(strHglt.m_iStartChar != -1 && strHglt.m_iEndChar != -1)
			{
				strHglt.m_rect = rStr;
				int len = strHglt.m_iEndChar - strHglt.m_iStartChar + 1;
				strHglt.m_str = startStr.Mid(strHglt.m_iStartChar, len);
				//dbg
				if(len > 4)
					int g  = 0;
				// Done - add to array.
				m_strHgltAr.Add(strHglt);
				// Add to clip
				clipRect |= strHglt.m_rect;
				return TRUE;
			}
			// Adjust char rect
			rChar.left = rChar.right;
		}
	}
	////////////////////////
	// Else multiple lines.
	else
	{
		CStrHglt strHglt;

		////////////////////////////////
		// Get the first selected line.
		//
		strHglt.m_str = startStr;
		strHglt.m_srcArElmt = startLineElmt;
		strHglt.m_rect = startStrRect;

		// If an empty string, or if starting after the 
		// string's end, save as empty.
		if(startStrLen == 0 || dragStartPt.x > startStrRect.right)
		{
			strHglt.m_str = _T("");							// ensure empty
			strHglt.m_rect.right = strHglt.m_rect.left; 	// ensure empty
			strHglt.m_isEmptyLine = TRUE;
			// Done - add to array.
			m_strHgltAr.Add(strHglt);
			// Add to clip
			clipRect |= strHglt.m_rect;

		}
		// Else take all chars after the start.
		else
		{
			CRect rectChar = startStrRect;
			CRect rectStr  = startStrRect;
			CSize sz;
			for(int i = 0; i < startStrLen; i++)
			{
				//dbg
				if(i == startStrLen - 1)
					int g = 0;

				sz = dc.GetTextExtent(startStr[i]);
				rectChar.right = rectChar.left + sz.cx;
				if(dragStartPt.x < rectChar.right)
				{
					rectStr.left = rectChar.left;
					int count = startStrLen - (i + 0);

					strHglt.m_iStartChar = i;
					strHglt.m_iEndChar = startStrLen - 1;;
					strHglt.m_str = startStr.Right(count);
					strHglt.m_rect = rectStr;
					break;
				}
				// Adjust char rect
				rectChar.left = rectChar.right;
			}

			// Done - add to array.
			if(strHglt.m_iStartChar != -1 && strHglt.m_iEndChar != -1 )
			{
				m_strHgltAr.Add(strHglt);
				clipRect |= strHglt.m_rect;
			}
			// Error
			else
			{
				return FALSE;
			}
		}// first line

		///////////////////////////////////
		// Get full lines between, if any.
		if(startLineElmt + 1 < endLineElmt)
		{
			CString str;
			CSize strSz;
			int lenStr;

			for(int i = startLineElmt + 1; i < endLineElmt; i++) // not the first or last/
			{
				str = srcStrAr[i];
				strSz = dc.GetTextExtent(str);
				lenStr = str.GetLength();
				
				strHglt.m_str		  = str;
				strHglt.m_srcArElmt   = i;
				strHglt.m_rect.top    = textOffset.cy + i * lineHgt;
				strHglt.m_rect.bottom = strHglt.m_rect.top + lineHgt;
				strHglt.m_rect.left   = textOffset.cx;
				strHglt.m_rect.right  = strHglt.m_rect.left + strSz.cx;

				// If an empty string.
				if(lenStr == 0)
				{
					strHglt.m_isEmptyLine = TRUE;
				}
				else
				{
					strHglt.m_iStartChar = 0;
					strHglt.m_iEndChar = lenStr - 1;
				}

				// Add to array.
				m_strHgltAr.Add(strHglt);
				// Add to clip
				clipRect |= strHglt.m_rect;
			}
		}
		/////////////////////
		// Get the end line.
		strHglt.m_str = endStr;
		strHglt.m_srcArElmt = endLineElmt;
		strHglt.m_rect = endStrRect;

		// If an empty string, or if starting before the 
		// string's start, save as empty.
		if(endStrLen == 0 || dragEndPt.x < endStrRect.left)
		{
			strHglt.m_str = _T("");							// ensure empty
			strHglt.m_rect.right = strHglt.m_rect.left; 	// ensure empty
			strHglt.m_isEmptyLine = TRUE;

			// Done - add to array.
			m_strHgltAr.Add(strHglt);
			// Add to clip
			clipRect |= strHglt.m_rect;

		}
		// Else if drag pt is right of string.
		else if(dragEndPt.x >= endStrRect.right)
		{
			strHglt.m_iStartChar = 0;				// string's start
			strHglt.m_iEndChar = endStrLen - 1;		// strings end

			// Done - add to array.
			m_strHgltAr.Add(strHglt);
			// Add to clip
			clipRect |= strHglt.m_rect;
		}
		// Else take all chars before the end.
		else
		{
			CRect rectChar = endStrRect;
			CRect rectStr  = endStrRect;
			CSize sz;
			for(int i = 0; i < endStrLen; i++)
			{
				sz = dc.GetTextExtent(endStr[i]);
				rectChar.right = rectChar.left + sz.cx;
				if(dragEndPt.x < rectChar.right)
				{
					rectStr.right = rectChar.right;

					strHglt.m_iStartChar = 0;				// string's start
					strHglt.m_iEndChar = i;					// chars tested
					strHglt.m_str = endStr.Left(i + 1);
					strHglt.m_rect = rectStr;
					break;
				}
				// Adjust char rect
				rectChar.left = rectChar.right;
			}

			// Done - add to array.
			if(strHglt.m_iStartChar != -1 && strHglt.m_iEndChar != -1 )
			{
				m_strHgltAr.Add(strHglt);
				clipRect |= strHglt.m_rect;
			}
			// Error
			else
			{
				return FALSE;
			}
		}// last line		
	}
	// Done.
	return TRUE;
}

// Called from CDataView::OnLButtonDown().
void CTextSel::SetStartDragPt(CPoint start, CRect & clip)
{
	// Get current clip.
	clip = GetClipRect();

	// Zero all.
	Zero();

	// Set start pt.
	m_dragStartPt = start;
}

// Called from CDataView::OnMouseMove().
// ValidEndPt() should be called first.
// Note dc param must have View font object selected in.
BOOL CTextSel::SetEndDragPt(CPoint end, const CStringArray & srcStrAr,
							CClientDC & dc, CSize textOffset, CRect & clip)
{
	// Ensure valid pts and sufficent drag distance.
	if(!ValidEndPt(end))
		return FALSE;

	// Set end pt.
	m_dragEndPt = end;

	// Build the highlighted strings.
	if(!SetHightlightStrings(srcStrAr, dc, textOffset, clip))
		return FALSE;

	// OK 
	return TRUE;
}

// Called from CDataView::OnMouseMove().
BOOL CTextSel::ValidEndPt(CPoint end)
{
	if( !PointIsEmpty(m_dragStartPt) && !PointIsEmpty(end) )
	{
		// If dragged a few pix from the start pt.
		if(abs(end.x - m_dragStartPt.x) > 5 ||
		   abs(end.y - m_dragStartPt.y) > 5 )
		{
			return TRUE;
		}
	}
	return FALSE;
}

// Draw the highlighted text.
// We use a back buffer here as redraws occure with
// each mouse move producing lots of flicker.
void CTextSel::DrawHighlight(CDC * pDC, LOGFONT lf)
{	
	CStrHglt strHglt;
	int nStrings = m_strHgltAr.GetSize();

	// dbg
	if(nStrings == 2)
		int g = 0;

	// Ensure.
	if(!nStrings)
		return;

	// Set text color and bk color.
	COLORREF oldTextColor = pDC->SetTextColor(RGB(255, 255, 255));	// white
	COLORREF oldBkColor = pDC->SetBkColor(RGB(0, 0, 128));			// navy

	// Draw each highlight string.
	for(int i = 0; i < nStrings; i++)
	{
		strHglt = m_strHgltAr[i];

		// Draw highlite string
		pDC->TextOut(strHglt.m_rect.left, strHglt.m_rect.top, strHglt.m_str);

	}
	// Restore colors.
	pDC->SetTextColor(oldTextColor);
	pDC->SetBkColor(oldBkColor);

/*
// crappy - no change in flicker - need whole view in bitmap?
// clip is painted errasing un-highlighted text
//////////////test buffer
	// Get a logical clip box.
	CRect clip;
	pDC->GetClipBox(&clip);	// MM_TEXT

	// If no Clip - nothing to do.
	if(clip.IsRectEmpty())
		return;

	// Create and prepare a compatible memory DC.
	CDC memDC;
	memDC.CreateCompatibleDC(pDC);
	memDC.SetMapMode(MM_TEXT);

	// Set up memdc font.
	if(!(HFONT)m_font)
		m_font.CreateFontIndirect(&lf);
	CFont * pOldFont = memDC.SelectObject(&m_font);

	// Clean bitmap if it already exists.
	if(m_screenBitmap.m_hObject)
		m_screenBitmap.DeleteObject();

	// Create and select a clip box sized bitmap.
	//
	// Note CreateCompBitmap() requires device units, which
	// are simply the logClip.top / logClip.bottom device 
	// units inverted. 
	m_screenBitmap.CreateCompatibleBitmap(pDC, 
										  clip.right  - clip.left,
										  clip.bottom - clip.top);	// MM_TEXT

	CBitmap * pOldBitmap = memDC.SelectObject(&m_screenBitmap);

	// Set the dc's window origin to the logical 
	// clip box's upper left.
	memDC.SetWindowOrg(clip.left, clip.top);

	// Draw highlight strings into the clip box sized bitmap.
	//
	COLORREF oldTextColor = memDC.SetTextColor(RGB(255, 255, 255));	// white
	COLORREF oldBkColor = memDC.SetBkColor(RGB(0, 0, 128));			// navy

	// bug - colors correctly, but errases all text
	// in clip before it gets highlighted
	memDC.SelectStockObject(WHITE_BRUSH);
	memDC.PatBlt(clip.left,
				 clip.top,
				 clip.Width(),
				 clip.Height(),
				 PATCOPY);


	// Draw each highlight string.
	for(int i = 0; i < nStrings; i++)
	{
		strHglt = m_strHgltAr[i];

		// Draw highlite string
		if(!strHglt.m_str.IsEmpty())
			memDC.TextOut(strHglt.m_rect.left, strHglt.m_rect.top, strHglt.m_str);

	}

	// Blast blit it to screen
	pDC->BitBlt(clip.left,    clip.top,
				clip.right  - clip.left,
				clip.bottom - clip.top, 
				&memDC,
				clip.left, clip.top, 
				SRCCOPY);
		
	// Restore colors.
	memDC.SetTextColor(oldTextColor);
	memDC.SetBkColor(oldBkColor);

	// Clean font
	memDC.SelectObject(pOldFont);

	// Select out bitmap
	memDC.SelectObject(pOldBitmap);
*/
}

CRect CTextSel::GetClipRect()
{
	// Get current highlighting rect, if any.
	CRect rect;
	rect.SetRectEmpty();
	int sz = m_strHgltAr.GetSize();
	for(int i = 0; i < sz; i++)
		rect |= m_strHgltAr[i].m_rect;

	return rect;
}

// Get selected text into str param, including newlines.
BOOL CTextSel::GetSelectedText(CString & str)
{
	int nStrings = m_strHgltAr.GetSize();

	// If none.
	if(!nStrings)
		return FALSE;

	CString temp;
	CString newline("\r\n");

	// Add each string to the str param.
	for(int i = 0; i < nStrings; i++)
	{
		temp = m_strHgltAr[i].m_str;	// May be an empty line

		// Remove any newline, space, and tab from begining and end.
		temp.TrimLeft();
		temp.TrimRight();

		// Add a space char to end if not the last.
		if(i < nStrings - 1)
			temp += newline;

		// Add to param
		str += temp;
	}

	// Ensure.
	if(!temp.IsEmpty())
		return TRUE;

	// Nada
	return FALSE;
}

BOOL CTextSel::PointInSelection(const CPoint & pt)
{
	int nStrings = m_strHgltAr.GetSize();

	// If none.
	if(!nStrings)
		return FALSE;

	// Check each string's rect for a hit.
	for(int i = 0; i < nStrings; i++)
	{
		if(m_strHgltAr[i].m_rect.PtInRect(pt))
			return TRUE;
	}
	return FALSE;
}




